//
//  BaseControllerActions.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 17/02/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import Foundation
import UIKit

class BaseControllerActions{
    var controller: UIViewController
    
    init(controller: UIViewController){
        self.controller = controller
    }
    
    let settings = NSUserDefaults.standardUserDefaults()
    
    func setTitleText(title: String){
        controller.navigationController?.navigationBar.topItem?.title = title;
    }
    
    func dismissView(){
        controller.dismissViewControllerAnimated(true, completion: nil)
    }
    
    func getSegueTarget(segue: UIStoryboardSegue)->UIViewController{
        let first = segue.destinationViewController as UIViewController
        if(first is UINavigationController){
            let navigationController = first as! UINavigationController
            return navigationController.viewControllers[0] as UIViewController
        }else{
            return first
        }
    }
    
    func showWelcomeScreen(context: WelcomePageControllerDelegate){
        let obj: AnyObject? = Settings.get(SettingsConstants.WELCOME_PAGE_DISMISSED)
        if(obj==nil || (obj as! Bool)==false){
            let ctrl = controller.storyboard?.instantiateViewControllerWithIdentifier("welcome-screen") as! WelcomePageController
            ctrl.delegate=context
            controller.presentViewController(ctrl, animated: true, completion: nil)
        }
    }
    
    func checkUserChange(){
        let welcomePageDismissed: AnyObject? = settings.objectForKey(SettingsConstants.WELCOME_PAGE_DISMISSED)
        if(Settings.syncEnabled && welcomePageDismissed != nil && (welcomePageDismissed as! Bool)==true){
            let currentUsername = settings.stringForKey(SettingsConstants.USERNAME)
            let lastUsername: AnyObject? = settings.objectForKey(SettingsConstants.LAST_LOGIN)
            NSLog("checking username %@ neni %@", (lastUsername as! String), String(currentUsername))
            if(/*lastUsername != nil && */(lastUsername as! String) != currentUsername){
                requestSyncBegin()
            }
        }
    }
    
    func requestSyncBegin(){
        let alert = UIAlertController(title: "Overwrite all data?", message: "A change of username detected. All data will be overwritten by data of the new user. Do you want to continue?", preferredStyle: UIAlertControllerStyle.Alert)
        alert.addAction(UIAlertAction(title: "Yes", style: UIAlertActionStyle.Default, handler: { action in
            // Overwrite data.
            self.runSyncBegin()
        }))
        alert.addAction(UIAlertAction(title: "No", style: UIAlertActionStyle.Default, handler: { action in
            // Change username back to the old one.
            let lastUsername = self.settings.stringForKey(SettingsConstants.LAST_LOGIN)
            self.settings.setValue(lastUsername, forKey: SettingsConstants.USERNAME)
        }))
        controller.presentViewController(alert, animated: true, completion: nil)
    }
    
    func runSyncBegin(){
        let ctrl = controller.storyboard?.instantiateViewControllerWithIdentifier("sync-begin") as! UINavigationController
        let target = ctrl.viewControllers[0] as! SyncBeginController
        target.delegate = controller as? SyncBeginControllerDelegate
        controller.presentViewController(ctrl, animated: true, completion: nil)
    }
    
    func setOfflineMode(on: Bool){
        settings.setValue(!on, forKey: SettingsConstants.SYNC_ENABLED)
    }
}
