//
//  DatabaseCollectionViewController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 17/02/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import Foundation
import CoreData
import UIKit

class DatabaseCollectionViewController: BaseCollectionViewController, NSFetchedResultsControllerDelegate, UpdateNotificationDelegate{
    
    var managedObjectContext: NSManagedObjectContext? = nil
    var storage: CentralStorage? = nil
    
    var updateNotification: UpdateNotification? = nil
    var databaseActions: DatabaseControllerActions? = nil
    
    override func viewDidLoad() {
        super.viewDidLoad()
        updateNotification = UpdateNotification(delegate: self)
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        if(databaseActions == nil){
            databaseActions = DatabaseControllerActions(storage: storage!, context: self)
        }
    }
    
    // MARK: - Actions
    
    func deleteSelected(sender: UIBarButtonItem){
        let paths = collectionView?.indexPathsForSelectedItems() as [NSIndexPath]?
        databaseActions?.deleteSelected(paths, sender: sender)
    }
    
    // MARK: - Table View
    
    func getAll()->NSFetchedResultsController?{
        return storage?.getAll()
    }
    
    func updateToolbarState(){
        
    }
    
    func configureCell(cell: UICollectionViewCell, atIndexPath: NSIndexPath){
        
    }
    
    override func numberOfSectionsInCollectionView(collectionView: UICollectionView) -> Int {
        return getAll()?.sections?.count ?? 0
    }
    
    override func collectionView(collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        let sectionInfo = (getAll()?.sections![section])! as NSFetchedResultsSectionInfo
        NSLog("count of projects is %@", String(sectionInfo.numberOfObjects))
        return sectionInfo.numberOfObjects
    }
    
    override func collectionView(collectionView: UICollectionView, cellForItemAtIndexPath indexPath: NSIndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCellWithReuseIdentifier("Cell", forIndexPath: indexPath) as UICollectionViewCell
        self.configureCell(cell, atIndexPath: indexPath)
        return cell
    }
    
    override func collectionView(collectionView: UICollectionView, didSelectItemAtIndexPath indexPath: NSIndexPath) {
        updateToolbarState()
    }
    
    override func collectionView(collectionView: UICollectionView, didDeselectItemAtIndexPath indexPath: NSIndexPath) {
        updateToolbarState()
    }
    
    override func collectionView(collectionView: UICollectionView, shouldSelectItemAtIndexPath indexPath: NSIndexPath) -> Bool {
        return true
    }
    
    override func setEditing(editing: Bool, animated: Bool) {
        super.setEditing(editing, animated: animated)
        updateToolbarState()
    }
    
    // MARK: - Reloading and updates
    internal func reloadData(){
        storage?.clearCache()
        //self.collectionView!.reloadItemsAtIndexPaths((self.collectionView?.indexPathsForVisibleItems())!)
        self.collectionView!.reloadData()
    }
    
    override func didChangeData() {
        reloadData()
    }
    
    // MARK: - Fetched results controller
    
    // I just implemented that with Swift. So I would like to share my implementation.
    // First initialise an array of NSBlockOperations:
    var blockOperations: [NSBlockOperation] = []
    
    
    // In the did change object method:
    func controller(controller: NSFetchedResultsController, didChangeObject anObject: AnyObject, atIndexPath indexPath: NSIndexPath?, forChangeType type: NSFetchedResultsChangeType, newIndexPath: NSIndexPath?) {
        
        if type == NSFetchedResultsChangeType.Insert {
            print("Insert Object: \(newIndexPath)")
            
            blockOperations.append(
                NSBlockOperation(block: { [weak self] in
                    if let this = self {
                        this.collectionView!.insertItemsAtIndexPaths([newIndexPath!])
                    }
                    })
            )
        }
        else if type == NSFetchedResultsChangeType.Update {
            print("Update Object: \(indexPath)")
            blockOperations.append(
                NSBlockOperation(block: { [weak self] in
                    if let this = self {
                        this.collectionView!.reloadItemsAtIndexPaths([indexPath!])
                    }
                    })
            )
        }
        else if type == NSFetchedResultsChangeType.Move {
            print("Move Object: \(indexPath)")
            
            blockOperations.append(
                NSBlockOperation(block: { [weak self] in
                    if let this = self {
                        this.collectionView!.moveItemAtIndexPath(indexPath!, toIndexPath: newIndexPath!)
                    }
                    })
            )
        }
        else if type == NSFetchedResultsChangeType.Delete {
            print("Delete Object: \(indexPath)")
            
            blockOperations.append(
                NSBlockOperation(block: { [weak self] in
                    if let this = self {
                        this.collectionView!.deleteItemsAtIndexPaths([indexPath!])
                    }
                    })
            )
        }
    }
    
    // In the did change section method:
    func controller(controller: NSFetchedResultsController, didChangeSection sectionInfo: NSFetchedResultsSectionInfo, atIndex sectionIndex: Int, forChangeType type: NSFetchedResultsChangeType) {
        
        if type == NSFetchedResultsChangeType.Insert {
            print("Insert Section: \(sectionIndex)")
            
            blockOperations.append(
                NSBlockOperation(block: { [weak self] in
                    if let this = self {
                        this.collectionView!.insertSections(NSIndexSet(index: sectionIndex))
                    }
                    })
            )
        }
        else if type == NSFetchedResultsChangeType.Update {
            print("Update Section: \(sectionIndex)")
            blockOperations.append(
                NSBlockOperation(block: { [weak self] in
                    if let this = self {
                        this.collectionView!.reloadSections(NSIndexSet(index: sectionIndex))
                    }
                    })
            )
        }
        else if type == NSFetchedResultsChangeType.Delete {
            print("Delete Section: \(sectionIndex)")
            
            blockOperations.append(
                NSBlockOperation(block: { [weak self] in
                    if let this = self {
                        this.collectionView!.deleteSections(NSIndexSet(index: sectionIndex))
                    }
                    })
            )
        }
    }
    
    // And finally, in the did controller did change content method:
    func controllerDidChangeContent(controller: NSFetchedResultsController) {
        collectionView!.performBatchUpdates({ () -> Void in
            for operation: NSBlockOperation in self.blockOperations {
                operation.start()
            }
            }, completion: { (finished) -> Void in
                self.blockOperations.removeAll(keepCapacity: false)
        })
    }
    
    // I personally added some code in the deinit method as well, in order to cancel the operations when the ViewController is about to get deallocated:
    deinit {
        // Cancel all block operations when VC deallocates
        for operation: NSBlockOperation in blockOperations {
            operation.cancel()
        }
        
        blockOperations.removeAll(keepCapacity: false)
    }
}