//
//  DatabaseControllerActions.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 23/02/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import Foundation
import CoreData
import UIKit

/// Common databsse actions used in view controllers.
class DatabaseControllerActions{
    
    var context: UIViewController
    var storage: CentralStorage
    
    init(storage: CentralStorage, context: UIViewController){
        self.storage = storage
        self.context = context
    }
    
    /**
     Converts an array of index paths to an array of Core Data objects.
     
     - parameter paths: Searched paths.
     
     - returns: An array with fetched objects.
     */
    func getItemsForPaths(paths: [NSIndexPath]?)->[NSManagedObject]{
        var result = [NSManagedObject]()
        if(paths != nil){
            for indexPath in paths!{
                let object = storage.get(indexPath)
                if(object != nil){
                    result.append(object!)
                }
            }
        }
        return result
    }
    
    /**
     Delete all items at the given paths. A confirmation dialog is shown first.
     
     - parameter paths:  Paths of objects to be deleted.
     - parameter sender: The button for attaching a confirmation dialog.
     */
    func deleteSelected(paths: [NSIndexPath]?, sender: UIBarButtonItem){
        let items = getItemsForPaths(paths)
        if(items.count > 0){
            let optionMenu = UIAlertController(title: String(format: l("Really delete %u items?"), paths!.count), message: l("All included content will be deleted too."), preferredStyle: .ActionSheet)
            
            let action = UIAlertAction(title: "Delete", style: .Destructive, handler: {
                (alert: UIAlertAction!) -> Void in
                self.deleteItems(items)
            })
            
            let cancelAction = UIAlertAction(title: "Cancel", style: .Default, handler: {
                (alert: UIAlertAction!) -> Void in
            })
            
            optionMenu.addAction(action)
            optionMenu.addAction(cancelAction)
            optionMenu.popoverPresentationController?.barButtonItem = sender
            
            context.presentViewController(optionMenu, animated: true, completion: nil)
        }
    }
    
    /**
     Deletes the given objects from Core Data and synchronises changes.
     
     - parameter items: Objects to be deleted.
     */
    func deleteItems(items: [NSManagedObject]){
        for object in items{
            if(object.managedObjectContext != nil){
                self.storage.delete(object)
            }
        }
        self.storage.synchronise()
    }
}
