//
//  DatabaseTableViewController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 10/7/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import Foundation
import CoreData
import UIKit

class DatabaseTableViewController: BaseTableViewController, NSFetchedResultsControllerDelegate, UpdateNotificationDelegate{
    
    var managedObjectContext: NSManagedObjectContext? = nil
    var storage: CentralStorage? = nil
    var frc: NSFetchedResultsController? = nil
    
    var updateNotification: UpdateNotification? = nil
    var databaseActions: DatabaseControllerActions? = nil
    
    override func viewDidLoad() {
        super.viewDidLoad()
        updateNotification = UpdateNotification(delegate: self)
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        if(databaseActions == nil){
            databaseActions = DatabaseControllerActions(storage: storage!, context: self)
        }
    }
    
    // MARK: - Actions
    
    func deleteSelected(sender: UIBarButtonItem){
        let paths = tableView?.indexPathsForSelectedRows as [NSIndexPath]?
        databaseActions?.deleteSelected(paths, sender: sender)
    }
    
    // MARK: - Table View
    
    func getAll()->NSFetchedResultsController?{
        if(frc == nil){
            frc = storage!.getAll()
        }
        return frc
    }
    
    func updateToolbarState(){
        
    }
    
    func configureCell(cell: UITableViewCell, atIndexPath: NSIndexPath){
        
    }
    
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return getAll()?.sections?.count ?? 0
    }
    
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        let sectionInfo = (getAll()?.sections![section])! as NSFetchedResultsSectionInfo
        return sectionInfo.numberOfObjects
    }
    
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("Cell", forIndexPath: indexPath) as UITableViewCell
        self.configureCell(cell, atIndexPath: indexPath)
        return cell
    }
    
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        updateToolbarState()
    }
    
    override func tableView(tableView: UITableView, didDeselectRowAtIndexPath indexPath: NSIndexPath) {
        updateToolbarState()
    }
    
    override func tableView(tableView: UITableView, canEditRowAtIndexPath indexPath: NSIndexPath) -> Bool {
        // Return false if you do not want the specified item to be editable.
        return true
    }
    
    override func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
        
    }
    
    override func setEditing(editing: Bool, animated: Bool) {
        super.setEditing(editing, animated: false)
        
        // Updates bottom bar action buttons appropriate to the current selected rows.
        updateToolbarState()
        
        // Reload data after user finishes edit mode. Reloading is not performed in edit mode.
        if(editing == false){
            reloadData()
        }
    }
    
    override func tableView(tableView: UITableView, didEndEditingRowAtIndexPath indexPath: NSIndexPath) {
        // Reload data after user finishes edit mode. Reloading is not performed in edit mode.
        tableView.reloadRowsAtIndexPaths(tableView.indexPathsForVisibleRows!, withRowAnimation: UITableViewRowAnimation.None)
        super.setEditing(false, animated: false)
        updateToolbarState()
    }
    
    // MARK: - Reloading and updates
    internal func reloadData(){
        storage?.clearCache() // delete obsolete records from cache
        frc = nil //  clear the current fetched results controller
        getAll() // fetch actual rows
        self.tableView.reloadData()
    }
    
    override func didChangeData() {
        // Do not reload data if user is currently in the edit mode. It could cause unpredictable behaviour.
        if(!self.tableView.editing){
            reloadData()
        }
    }
    
    // MARK: - Fetched results controller
    func controllerWillChangeContent(controller: NSFetchedResultsController) {
        getAll() // fetch rows if not fetched
        self.tableView.beginUpdates()
    }
    
    func controller(controller: NSFetchedResultsController, didChangeSection sectionInfo: NSFetchedResultsSectionInfo, atIndex sectionIndex: Int, forChangeType type: NSFetchedResultsChangeType) {
        switch type {
        case .Insert:
            self.tableView.insertSections(NSIndexSet(index: sectionIndex), withRowAnimation: .Fade)
        case .Delete:
            self.tableView.deleteSections(NSIndexSet(index: sectionIndex), withRowAnimation: .Fade)
        default:
            return
        }
    }
    
    func controller(controller: NSFetchedResultsController, didChangeObject anObject: AnyObject, atIndexPath indexPath: NSIndexPath?, forChangeType type: NSFetchedResultsChangeType, newIndexPath: NSIndexPath?) {
        switch type {
        case .Insert:
            tableView.insertRowsAtIndexPaths([newIndexPath!], withRowAnimation: .Fade)
        case .Delete:
            tableView.deleteRowsAtIndexPaths([indexPath!], withRowAnimation: .Fade)
        case .Update:
            if let cell = tableView.cellForRowAtIndexPath(indexPath!){
                self.configureCell(cell, atIndexPath: indexPath!)
            }
        case .Move:
            tableView.deleteRowsAtIndexPaths([indexPath!], withRowAnimation: .Fade)
            tableView.insertRowsAtIndexPaths([newIndexPath!], withRowAnimation: .Fade)
        }
    }
    
    func controllerDidChangeContent(controller: NSFetchedResultsController) {
        self.tableView.endUpdates()
    }
}