//
//  DefaultFormValidator.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 9/11/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import Foundation
import SwiftValidator

protocol DefaultFormValidatorDelegate{
    func validationSuccessful()
}

class DefaultFormValidator: Validator, ValidationDelegate{
    
    var delegate: DefaultFormValidatorDelegate? = nil
    
    override init(){
        super.init()
        styleTransformers(success: textFieldOK, error: textFieldError)
    }
    
    func validate(defaultFormDelegate: DefaultFormValidatorDelegate) {
        self.delegate=defaultFormDelegate
        super.validate(self)
    }
    
    func validationSuccessful() {
        self.delegate?.validationSuccessful()
    }
    
    func requiredField(field: UITextField){
        self.registerField(field, rules: [RequiredRule()])
    }
    
    func validationFailed(errors:[UITextField:ValidationError]) {
        // turn the fields to red
        for (field, error) in self.errors {
            field.layer.borderColor = UIColor.redColor().CGColor
            field.layer.borderWidth = 1.0
            error.errorLabel?.text = error.errorMessage // works if you added labels
            error.errorLabel?.hidden = false
        }
    }

    func textFieldOK(rule: ValidationRule){
        /*let field = rule.textField
        field.layer.borderColor = UIColor.greenColor().CGColor
        field.layer.borderWidth = 1.0
        rule.errorLabel?.hidden = true*/
    }
    
    func textFieldError(error: ValidationError){
        let field = error.textField
        field.layer.borderColor = UIColor.redColor().CGColor
        field.layer.borderWidth = 1.0
        error.errorLabel?.text = error.errorMessage // works if you added labels
        error.errorLabel?.hidden = false
    }
    
}
