//
//  DownloadingPictureController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 10/02/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import Foundation
import UIKit
import SwiftyJSON

protocol DownloadPictureControllerDelegate{
    func didDownloadedPicture(picture: UIImage)
}

class DownloadPictureController: UIViewController, UIPopoverPresentationControllerDelegate{
    
    var delegate: DownloadPictureControllerDelegate? = nil
    var featurePictureStorage: FeaturePictureStorage? = nil
    var feature: Feature? = nil
    
    var picture: UIImage? = nil
    var request: ServerRequest? = nil
    
    override func viewDidLoad() {
        startDownload()
    }
    
    func startDownload(){
        if(feature != nil && featurePictureStorage != nil){
            // create object
            let picture = featurePictureStorage?.newEntity() as! FeaturePicture
            picture.feature = feature
            let url = featurePictureStorage!.getItemURL(picture)
            
            // make request and save data
            request = ServerRequest(method: HTTPMethod.GET, url: url, bodyString: "")
            request?.runWithoutJSON({ (request, response, object) -> Void in
                //let json = JSON(object)
                let picture = UIImage(data: object as! NSData)
                //let str = self.request?.operation?.responseString
                //NSLog("response json = %@", str!)
                self.downloadFinished(picture)
                }, failure: { (request, response, error, object) -> Void in
                    let alert = UIAlertController(title: l("Error"), message: l("Feature picture cannot be downloaded. Please check your internet connection or contact server admin."), preferredStyle: UIAlertControllerStyle.Alert)
                    alert.addAction(UIAlertAction(title: "OK", style: UIAlertActionStyle.Default, handler: { action in
                        self.dismissViewControllerAnimated(true, completion: nil)
                    }))
                    self.presentViewController(alert, animated: true, completion: nil)
            })
        }
    }
    
    func downloadFinished(picture: UIImage?){
        if(picture != nil){
            delegate?.didDownloadedPicture(picture!)
        }
        dismissViewControllerAnimated(true, completion: nil)
    }
    
    func stopDownload(){
        request?.stop()
        dismissViewControllerAnimated(true, completion: nil)
    }
    
    @IBAction func cancelClick(sender: AnyObject) {
        stopDownload()
    }
    
    func popoverPresentationControllerShouldDismissPopover(popoverPresentationController: UIPopoverPresentationController) -> Bool{
        return false
    }
    
    func popoverControllerShouldDismissPopover(popoverController: UIPopoverController) -> Bool{
        return false
    }
}
