//
//  FeatureAttributeStorage.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 1/11/16.
//  Copyright (c) 2016 MENDELU. All rights reserved.
//

import Foundation
import CoreData
import UIKit
import SwiftyJSON

class FeatureAttributeStorage: CentralStorage{
    
    let ENTITY_NAME = FeatureAttribute.getEntityName()
    
    required init(moc: NSManagedObjectContext, delegate: NSFetchedResultsControllerDelegate? = nil) {
        super.init(moc: moc, delegate: delegate, entityName: ENTITY_NAME)
        
        // override data mapping
        self.mapping = FeatureAttributeDataMapping(localStorage: local)
    }
    
    override func getCreateURL(item: NSManagedObject) -> String {
        let attr = item as! FeatureAttribute
        let feature = attr.feature
        let layer = attr.feature.layer
        let featureId = feature.name
        return "/layer/"+layer.scheme+"/"+layer.name+"/feature/"+featureId+"/attribute"
    }
    
    override func getItemURL(item: NSManagedObject) -> String {
        let object = item as! FeatureAttribute
        let layer = object.feature.layer
        return "/layer/"+layer.scheme+"/"+layer.name+"/feature/"+object.feature.name+"/attribute/"+object.layerattribute.name
    }
    
    override func getItemByURLParts(urlParts: [String]) -> NSManagedObject? {
        if((urlParts.count)==8){
            return getByLayerAndFeatureAndAttribute(urlParts[3], featureName: urlParts[5], attrName: urlParts[7])
        }
        return nil
    }
    
    override func getMapping() -> NSDictionary? {
        return [
            "hash": "hashString",
            "name": "layerattribute.name",
            "value": "valueString",
        ]
    }
    
    override func getPostMapping() -> NSDictionary? {
        // we need to send also attribute name in POST request
        return getMapping()
    }
    
    override func getOutputMapping() -> NSDictionary? {
        return [
            "hash": "hashString",
            "value": "valueString",
        ]
    }
    
    override func getRelationsURLMapping() -> NSDictionary? {
        return [
            5: "Feature",
            7: "LayerAttribute"
        ]
    }
    
    override func getRelationsAttrMapping() -> NSDictionary? {
        return [
            "name": "LayerAttribute"
        ]
    }
    
    override func newEntity() -> NSManagedObject {
        let item = super.newEntity() as! FeatureAttribute
        return item
    }
    
    override func getDefaultSort() -> [NSSortDescriptor] {
        return [NSSortDescriptor(key: "layerattribute.title", ascending: true)]
    }
    
    func getAllForFeature(feature: Feature)->NSFetchedResultsController{
        let predicate = NSPredicate(format: "("+getDefaultPredicate()+") AND (feature = %@)", feature)
        return local.getAll(entityName, predicate: predicate, sort: getDefaultSort(), delegate: delegate)
    }
    
    func getByLayerAndFeatureAndAttribute(layerName: String, featureName: String, attrName: String)->FeatureAttribute?{
        let predicate = NSPredicate(format: "(feature.name=%@) AND (layerattribute.name=%@) AND (layerattribute.layer.name = %@)", featureName, attrName, layerName)
        let items = local.getAll(entityName, predicate: predicate, sort: getDefaultSort(), delegate: delegate)
        if(items.fetchedObjects?.count > 0){
            return items.fetchedObjects?.first as! FeatureAttribute?
        }
        return nil
    }

}
