//
//  FeatureInfoController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 18/01/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import Foundation
import UIKit
import SwiftyJSON
import GoogleMaps

protocol FeatureInfoControllerDelegate{
    func didChangeCoordinates(tempGeometry: Geometry?)
}

class FeatureInfoController: UITableViewController, DefaultFormValidatorDelegate{
    
    var feature: Feature? = nil
    var layerInProject: LayerInProject? = nil
    var tempGeometry: Geometry? = nil
    var tempPrecision: Double? = nil
    var newFeatureCreated: NSDate? = nil
    var delegate: FeatureInfoControllerDelegate? = nil
    
    var validator: DefaultFormValidator? = nil
    private var isPointType: Bool? = nil
    
    @IBOutlet weak var latitudeTextField: UITextField!
    @IBOutlet weak var longitudeTextField: UITextField!
    
    @IBOutlet weak var minLatText: UILabel!
    @IBOutlet weak var maxLatText: UILabel!
    @IBOutlet weak var minLonText: UILabel!
    @IBOutlet weak var maxLonText: UILabel!

    @IBOutlet weak var lengthText: UILabel!
    @IBOutlet weak var areaText: UILabel!
    
    @IBOutlet weak var precisionText: UILabel!
    
    @IBOutlet weak var createdText: UILabel!
    @IBOutlet weak var modifiedText: UILabel!
    
    override func viewDidLoad() {
        loadValues()
        initValidation()
    }
    
    func loadValues(){
        if(layerInProject!.layer!.kind == LayerTypes.point){
            // load point values
            latitudeTextField.text = String(tempGeometry!.latitude)
            longitudeTextField.text = String(tempGeometry!.longitude)
            
        }else{
            // load line / polygon values
            
            // bounding box
            let bbox = tempGeometry!.boundingBox
            minLatText.text = bbox[0].stringValue
            minLonText.text = bbox[1].stringValue
            maxLatText.text = bbox[2].stringValue
            maxLonText.text = bbox[3].stringValue
            
            // length
            let path = tempGeometry!.path
            let length = GMSGeometryLength(path)
            lengthText.text = String(format: "%.3f", length) + " m"
            
            // area
            if(layerInProject!.layer!.kind == LayerTypes.polygon){
                let area = GMSGeometryArea(path)
                let str = String(format: "%.3f", area) + " m2"
                // make super index for square meters
                let font:UIFont? = UIFont(name: "Helvetica", size:17)
                let fontSuper:UIFont? = UIFont(name: "Helvetica", size:10)
                let attString:NSMutableAttributedString = NSMutableAttributedString(string: str, attributes: [NSFontAttributeName:font!])
                attString.setAttributes([NSFontAttributeName:fontSuper!,NSBaselineOffsetAttributeName:10], range: NSRange(location:str.characters.count-1,length:1))
                areaText.attributedText = attString;
            }
        }
        
        // load common values
        if(tempPrecision == nil){
            precisionText.text = l("Unknown")
        }else{
            precisionText.text = String(tempPrecision!)
        }
        
        if(feature == nil){
            createdText.text = dateToJSON(newFeatureCreated!)
            modifiedText.text = "-"
        }else{
            createdText.text = dateToJSON(feature!.created)
            modifiedText.text = dateToJSON(feature!.lastEdit)
        }
        
    }
    
    private func initValidation(){
        validator = DefaultFormValidator()
        if(layerInProject?.layer?.kind == LayerTypes.point.lowercaseString){
            validator!.requiredField(latitudeTextField)
            validator!.requiredField(longitudeTextField)
        }
    }
    
    func getFilledCoordinates()->[Double]{
        return [
            Double(latitudeTextField.text!)!,
            Double(longitudeTextField.text!)!
            ]
    }
    
    @IBAction func doneClick(sender: AnyObject) {
        validator?.validate(self)
    }
    
    func validationSuccessful() {
        // send back changed coordinates
        if(layerInProject?.layer?.kind == LayerTypes.point.lowercaseString){
            let new = getFilledCoordinates()
            if(new[0] != tempGeometry?.latitude || new[1] != tempGeometry?.longitude){
                tempGeometry = Geometry(json: JSON(new), type: LayerTypes.point)
                delegate?.didChangeCoordinates(tempGeometry)
            }
        }
        // return back to feature detail
        navigationController?.popViewControllerAnimated(true)
    }
    
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if(layerInProject!.layer!.kind == LayerTypes.line && section == 1){
            return 1 // line does not have an area, hide it
        }else{
            return super.tableView(tableView, numberOfRowsInSection: section)
        }
    }
}
