//
//  FeaturePicture.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 09/02/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import Foundation
import CoreData
import UIKit

@objc(FeaturePicture)
class FeaturePicture: Model, ModelProtocol {
    
    override class func getEntityName() -> String {
        return "FeaturePicture"
    }

    override func getEntityName() -> String {
        return "FeaturePicture"
    }
    
    @NSManaged var hashString: String
    @NSManaged var hidden: Bool
    @NSManaged var path: String
    @NSManaged var mergeFlag: Int64
    
    @NSManaged var feature: Feature?
    
    // MARK: - Image
    
    /// Default path for saving feature pictures.
    static let PICTURES_PATH = "feature-pictures"
    
    /// Extension for image files.
    static let PICTURE_EXT = "jpg"
    
    /// Dynamic property for loading and saving feature image in the file system instead of saving to the database.
    var image: UIImage?{
        get {
            if(feature == nil){
                return nil
            }else{
                let fileManager = FileManager()
                return fileManager.imageForPath(path)
            }
        }
        set (image) {
            if(feature != nil){
                // check target dir and create if not exists
                let fileManager = FileManager()
                fileManager.makeDirectory(imageDir)
                
                if(image != nil){
                    // save image
                    let compression = CGFloat(config.featurePictureSize.jpegCompression)
                    fileManager.saveImageAsJPEG(image!, path: imagePath, compression: compression)
                    
                    // save file path to db
                    self.path = imagePath
                    
                }else{
                    // delete image if image is being set to nil
                    if(path != ""){
                        fileManager.delete(path)
                        self.path = ""
                    }
                }
            }
        }
    }
    
    /// Dynamic property for loading the feature picture as NSData.
    var imageData: NSData?{
        get {
            if(feature == nil){
                return nil
            }else{
                let fileManager = FileManager()
                return fileManager.fileForPath(path)
            }
        }
    }
    
    /// Dynamic property for generating image directory.
    private var imageDir: String{
        get {
            return feature!.layer.featuresPath
        }
    }
    
    /// Dynamic property for generating image path including imageDir.
    private var imagePath: String{
        get {
            return imageDir + feature!.localName + "." + FeaturePicture.PICTURE_EXT
        }
    }
    
    /**
    After deleting object in CoreData delete the image file too.
    */
    override func didSave() {
        if(self.deleted && path != ""){
            let fileManager = FileManager()
            fileManager.delete(path)
        }
        super.didSave()
    }
    
    /**
     Updates feature's hasPicture property if FeaturePicture is going to be deleted.
     
     - parameter key: Changed property key.
     */
    override func didChangeValueForKey(key: String) {
        if(key == "hidden"){
            feature?.hasPicture = !hidden
        }
        super.didChangeValueForKey(key)
    }
}
