//
//  FeaturePictureStorage.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 09/02/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import Foundation
import CoreData
import UIKit
import SwiftyJSON

class FeaturePictureStorage: CentralStorage{
    
    let ENTITY_NAME = FeaturePicture.getEntityName()
    
    required init(moc: NSManagedObjectContext, delegate: NSFetchedResultsControllerDelegate? = nil) {
        super.init(moc: moc, delegate: delegate, entityName: ENTITY_NAME)
    }
    
    override func getCreateURL(item: NSManagedObject) -> String {
        let picture = item as! FeaturePicture
        let feature = picture.feature
        let layer = feature!.layer
        return "/layer/"+layer.scheme+"/"+layer.name+"/feature/"+feature!.name+"/picture"
    }
    
    override func getItemURL(item: NSManagedObject) -> String {
        let picture = item as! FeaturePicture
        let feature = picture.feature
        let layer = feature!.layer
        return "/layer/"+layer.scheme+"/"+layer.name+"/feature/"+feature!.name+"/picture"
    }
    
    override func getItemByURLParts(urlParts: [String]) -> NSManagedObject? {
        if((urlParts.count)==7){
            return getByFeatureAndLayer(urlParts[5], layerName: urlParts[3])
        }
        return nil
    }
    
    override func getMapping() -> NSDictionary? {
        return [
            "hash": "hashString"
        ]
    }
    
    override func getRelationsURLMapping() -> NSDictionary? {
        return [
            5: "Feature"
        ]
    }
    
    override func newEntity() -> NSManagedObject {
        let item = super.newEntity() as! FeaturePicture
        return item
    }
    
    override func getDefaultSort() -> [NSSortDescriptor] {
        return [NSSortDescriptor(key: "hashString", ascending: true)]
    }
    
    func getByFeatureAndLayer(featureName: String, layerName: String)->FeaturePicture?{
        let predicate = NSPredicate(format: "("+getDefaultPredicate()+") AND (feature.name = %@) AND (feature.layer.name = %@)", featureName, layerName)
        let all = local.getAll(entityName, predicate: predicate, sort: getDefaultSort(), delegate: delegate)
        if(all.fetchedObjects == nil || all.fetchedObjects?.count == 0){
            return nil
        }else{
            return all.fetchedObjects![0] as? FeaturePicture
        }
    }
    
    override func preparePostRequestData(event: Event, item: NSManagedObject?) -> SyncRequestData? {
        var data = super.preparePostRequestData(event, item: item)
        // insert the image instead of JSON body
        if let picture = item as? FeaturePicture{
            data!.body = picture.imageData
        }
        return data
    }
    
    override func preparePutRequestData(event: Event, item: NSManagedObject?) -> SyncRequestData? {
        var data = super.preparePostRequestData(event, item: item)
        // insert the image instead of JSON body
        if let picture = item as? FeaturePicture{
            data!.body = picture.imageData
        }
        return data
    }
    
    override func handleIncomingPutEvent(event: JSON) {
        super.handleIncomingPutEvent(event)
        // delete local image after change on server
        let item = getItemByURL(event["uri"].stringValue) as? FeaturePicture
        item?.image = nil
    }
}
