//
//  FeatureStorage.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 1/11/16.
//  Copyright (c) 2016 MENDELU. All rights reserved.
//

import Foundation
import CoreData
import UIKit
import SwiftyJSON

class FeatureStorage: CentralStorage{
    
    let ENTITY_NAME = Feature.getEntityName()
    
    required init(moc: NSManagedObjectContext, delegate: NSFetchedResultsControllerDelegate? = nil) {
        super.init(moc: moc, delegate: delegate, entityName: ENTITY_NAME)
    }
    
    override func getCreateURL(item: NSManagedObject) -> String {
        let feature = item as! Feature
        let layer = feature.layer
        return "/layer/"+layer.scheme+"/"+layer.name+"/feature"
    }
    
    override func getItemURL(item: NSManagedObject) -> String {
        let object = item as! Feature
        return "/layer/"+object.layer.scheme+"/"+object.layer.name+"/feature/"+object.name
    }
    
    override func getItemByURLParts(urlParts: [String]) -> NSManagedObject? {
        if((urlParts.count)==6){
            return getByLayerAndFeature(urlParts[3], featureName: urlParts[5])
        }
        return nil
    }
    
    override func getMapping() -> NSDictionary? {
        return [
            "hash": "hashString",
            "id": "name", // We interpret id as name for unity with other entities.
            "properties": [
                "has_picture": "hasPicture",
                "created": "created",
                "last_edit": "lastEdit",
                "precision": "precision"
            ],
            "geometry": [
                "type": "type",
                "coordinates": "geometry"
            ],
            "bbox": "bbox"
        ]
    }
    
    override func getRelationsURLMapping() -> NSDictionary? {
        return [
            3: "Layer"
        ]
    }
    
    override func newEntity() -> NSManagedObject {
        let item = super.newEntity() as! Feature
        item.created = NSDate()
        item.lastEdit = NSDate()
        return item
    }
    
    override func getDefaultSort() -> [NSSortDescriptor] {
        return [NSSortDescriptor(key: "name", ascending: true)]
    }
    
    func getAllForLayer(layer: Layer)->NSFetchedResultsController{
        let predicate = NSPredicate(format: "("+getDefaultPredicate()+") AND (layer = %@)", layer)
        return local.getAll(entityName, predicate: predicate, sort: getDefaultSort(), delegate: delegate)
    }
    
    func getByLayerAndFeature(layerName: String, featureName: String)->Feature?{
        let predicate = NSPredicate(format: "(layer.name = %@) AND (name=%@)", layerName, featureName)
        let items = local.getAll(entityName, predicate: predicate, sort: getDefaultSort(), delegate: delegate)
        if(items.fetchedObjects?.count > 0){
            return items.fetchedObjects?.first as! Feature?
        }
        return nil
    }
    
    override func getByKey(urlParts: [String], index: Int) -> NSManagedObject? {
        // feature name is not unique, filter by layer too
        return getByLayerAndFeature(urlParts[3], featureName: urlParts[5])
    }
    
}
