//
//  FileManager.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 11/02/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import Foundation
import UIKit

/// Class for manipulating files and images in the application documents directory.
class FileManager{
    
    func imageForPath(path: String)->UIImage?{
        let absolutePath = getDocumentsDirectory().stringByAppendingPathComponent(path)
        let image = UIImage(contentsOfFile: absolutePath)
        return image
    }
    
    func fileForPath(path: String)->NSData?{
        let absolutePath = getDocumentsDirectory().stringByAppendingPathComponent(path)
        let data = NSData(contentsOfFile: absolutePath)
        return data
    }
    
    func saveImageAsJPEG(image: UIImage, path: String, compression: CGFloat)->Bool{
        let jpegImageData = UIImageJPEGRepresentation(image, compression)
        let absolutePath = getDocumentsDirectory().stringByAppendingPathComponent(path)
        let result = jpegImageData!.writeToFile(absolutePath, atomically: true)
        return result
    }
    
    func getDocumentsDirectory()->AnyObject{
        let paths = NSSearchPathForDirectoriesInDomains(NSSearchPathDirectory.DocumentDirectory, NSSearchPathDomainMask.UserDomainMask, true)
        let documentsDirectory: AnyObject = paths[0]
        return documentsDirectory
    }
    
    func makeDirectory(path: String){
        let dataPath = getDocumentsDirectory().stringByAppendingPathComponent(path)
        do {
            try NSFileManager.defaultManager().createDirectoryAtPath(dataPath, withIntermediateDirectories: true, attributes: nil)
        } catch let error as NSError {
            print(error.localizedDescription);
        }
    }
    
    /**
     Deletes a file or a folder if exists. It deletes not empty folders too.
     
     - parameter path: Path of the file / folder to be delted. The path is relative to the app documents directory.
     */
    func delete(path: String){
        let absolutePath = getDocumentsDirectory().stringByAppendingPathComponent(path)
        if(NSFileManager.defaultManager().fileExistsAtPath(absolutePath)){
            do {
                try NSFileManager.defaultManager().removeItemAtPath(absolutePath)
            } catch let error as NSError {
                print(error.localizedDescription);
            }
        }
    }
    
}
