//
//  ViewController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 9/6/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import UIKit
import SwiftValidator

protocol FirstLoginControllerDelegate{
    func syncBeginSuccessful()
}

class FirstLoginViewController: UITableViewController, DefaultFormValidatorDelegate, SyncBeginControllerDelegate {
    
    var delegate:SyncBeginControllerDelegate? = nil
    var settings: NSUserDefaults=NSUserDefaults.standardUserDefaults()
    var validator: DefaultFormValidator? = nil
    
    @IBOutlet weak var loginField: UITextField!
    @IBOutlet weak var pwField: UITextField!
    @IBOutlet weak var badLoginCell: UITableViewCell!
    
    var didBadLogin = false
    
    override func viewDidLoad() {
        super.viewDidLoad()
        fillInDefaultValues()
        initValidation()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    private func initValidation(){
        validator = DefaultFormValidator()
        validator!.requiredField(loginField)
        validator!.requiredField(pwField)
    }
    
    @IBAction func doneClick(sender: AnyObject) {
        validator?.validate(self)
    }
    
    @IBAction func cancelClick(sender: AnyObject) {
        self.dismissViewControllerAnimated(true, completion: { () -> Void in
            
        })
    }
    
    func validationSuccessful() {
        saveLoginDetails()
        performSegueWithIdentifier("firstLoginDone", sender: self)
    }
    
    private func fillInDefaultValues(){
        if let login = settings.stringForKey(SettingsConstants.USERNAME){
            loginField.text=login
        }
        
        if let pw = settings.stringForKey(SettingsConstants.PASSWORD){
            pwField.text=pw
        }
    }
    
    private func saveLoginDetails(){
        if(!loginField.text!.isEmpty){
            settings.setValue(loginField.text!.trim, forKey: SettingsConstants.USERNAME)
        }
        if(!pwField.text!.isEmpty){
            settings.setValue(pwField.text, forKey: SettingsConstants.PASSWORD)
        }
    }
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if(segue.identifier=="firstLoginDone"){
            // pass delegate to the next controller
            let navigationController = segue.destinationViewController as! UINavigationController
            let target = navigationController.viewControllers[0] as! SyncBeginController
            target.delegate=self
        }
    }
    
    func syncBeginSuccessful() {
        self.delegate?.syncBeginSuccessful()
    }
    
    func badLogin(ctrl: SyncBeginController) {
        //self.navigationController?.popToRootViewControllerAnimated(true)
        dismissViewControllerAnimated(true, completion: nil)
        badLoginCell.hidden = false
    }

}

