//
//  Geometry.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 04/02/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import Foundation
import SwiftyJSON
import GoogleMaps

class Geometry{
    
    var data: JSON
    var type: String = "undefined"
    
    init(){
        data = JSON([])
        type = "undefined"
    }
    
    init(data: NSDictionary, type: String){
        self.data = JSON(data)
        self.type = type
    }
    
    init(string: String, type: String){
        data = stringToJSON(string)
        self.type = type
    }
    
    init(json: JSON, type: String){
        data = json
        self.type = type
    }
    /*
    func detectType()->String{
        if(isValidPoint(data)){
            type = LayerTypes.point
        }else if(isValidLine(data)){
            type = LayerTypes.line
        }else if(isValidPolygon(data)){
            type = LayerTypes.polygon
        }
        return type
    }*/
    
    var array: [[Double]] {
        get{
            var result = [[Double]]()
            var geom: JSON = []
            
            if(type == LayerTypes.line){
                geom = data
            }else if(type == LayerTypes.polygon){
                geom = data[0]
            }
            for (_, item) in geom{
                result.append([item[0].doubleValue, item[1].doubleValue])
            }
            return result
        }
    }
    
    func isValidPoint()->Bool{
        return isValidPoint(data)
    }
    
    func isValidPoint(json: JSON)->Bool{
        return json.count == 2 && json[0].double != nil && json[1].double != nil
    }
    /*
    func isValidLine(json: JSON)->Bool{
        var valid = false
        if(json[0] != nil && json[0][0].double != nil){
            valid = true
            for (_, item) in json[0]{
                if(!isValidPoint(item)){
                    valid = false
                }
            }
        }
        return valid
    }
    
    func isValidPolygon(json: JSON)->Bool{
        return json[0] != nil && isValidLine(json[0])
    }*/
    
    var path: GMSPath{
        get{
            let result = GMSMutablePath()
            for (item) in array{
                if(item.count == 2){
                    result.addLatitude(item[0], longitude: item[1])
                }else{
                    warning("MapManager:getPath() - Unrecognizable geometry received!")
                }
            }
            return result
        }
    }
    
    func append(coordinates: [Double]){
        if(coordinates.count == 2){
            if(type == LayerTypes.line){
                var geom = data.arrayValue
                geom.append(JSON(coordinates))
                data = JSON(geom)
            }else if(type == LayerTypes.polygon){
                if(data.count == 0){
                    data = [[]]
                }
                var geom = data[0].arrayValue
                geom.append(JSON(coordinates))
                data[0] = JSON(geom)
            }
        }
    }
    
    func popLast(){
        if(type == LayerTypes.line){
            var geom = data.arrayValue
            geom.popLast()
            data = JSON(geom)
        }else if(type == LayerTypes.polygon){
            var geom = data[0].arrayValue
            geom.popLast()
            data[0] = JSON(geom)
        }
    }
    
    var latitude: Double{
        get{
            return data[0].doubleValue
        }
    }
    
    var longitude: Double{
        get{
            return data[1].doubleValue
        }
    }
    
    func correctPolygonEnclosure(){
        if(type == LayerTypes.polygon){
            let geom = data[0]
            var arr = geom.arrayValue
            arr.append(geom[0])
            data[0] = JSON(arr)
        }
    }
    
    var boundingBox: JSON{
        get {
            var geom:JSON = []
            
            if(type == LayerTypes.point){
                return [latitude, longitude, latitude, longitude]
            }else if(type == LayerTypes.line){
                geom = data
            }else if(type == LayerTypes.polygon){
                geom = data[0]
            }
            
            var minLat = 90.0
            var minLon = 180.0
            var maxLat = 0.0
            var maxLon = -180.0
            
            for (_, item) in geom{
                if(item[0].doubleValue < minLat){
                    minLat = item[0].doubleValue
                }
                if(item[1].doubleValue < minLon){
                    minLon = item[1].doubleValue
                }
                if(item[0].doubleValue > maxLat){
                    maxLat = item[0].doubleValue
                }
                if(item[1].doubleValue > maxLon){
                    maxLon = item[1].doubleValue
                }
            }
            
            return JSON([minLat, minLon, maxLat, maxLon])
        }
    }
    
    var boundingBoxCenter: CLLocationCoordinate2D{
        get {
            let bbox = boundingBox
            return CLLocationCoordinate2D(latitude: (bbox[0].doubleValue+bbox[2].doubleValue)/2.0, longitude: (bbox[1].doubleValue+bbox[3].doubleValue)/2.0)
        }
    }
}
