//
//  ColorizeImage.swift
//  Gisella
//
//  Created by Rob Ryan, Jul 27 at 20:11
//  http://stackoverflow.com/a/31661519
//
//  Edited by Ivo Pisarovic on 10/9/15.
//

import Foundation
import UIKit

class ImageColorizer{
    func replacePixelsInImage(inputImage: UIImage, search: UIColor, replacement: UIColor, keepAlpha: Bool = true) -> UIImage {
        let inputCGImage     = inputImage.CGImage
        let colorSpace       = CGColorSpaceCreateDeviceRGB()
        let width            = CGImageGetWidth(inputCGImage)
        let height           = CGImageGetHeight(inputCGImage)
        let bytesPerPixel    = 4 as Int
        let bitsPerComponent = 8 as Int
        let bytesPerRow      = bytesPerPixel * width
        let bitmapInfo       = CGImageAlphaInfo.PremultipliedLast.rawValue
        
        let searchComponents = getColorComponents(search)
        let replacementComponents = getColorComponents(replacement)
        
        let context:CGContextRef = CGBitmapContextCreate(nil, width, height, bitsPerComponent, bytesPerRow, colorSpace, bitmapInfo)!
        CGContextDrawImage(context, CGRectMake(0, 0, CGFloat(width), CGFloat(height)), inputCGImage)
        
        
        let pixelBuffer = UnsafeMutablePointer<UInt32>(CGBitmapContextGetData(context))
        var currentPixel = pixelBuffer
        
        for var i = 0; i < Int(height); i++ {
            for var j = 0; j < Int(width); j++ {
                let pixel = currentPixel.memory
                //NSLog("pixel %u %u %u %u", red(pixel), green(pixel), blue(pixel), alpha(pixel))
                if isEqual(pixel, color: searchComponents) {
                    var newAlpha = replacementComponents.alpha
                    if(keepAlpha){
                        newAlpha = alpha(pixel)
                    }
                    currentPixel.memory = getColorBytes(
                        red: max(replacementComponents.red, 255-newAlpha),
                        green: max(replacementComponents.green, 255-newAlpha),
                        blue: max(replacementComponents.blue, 255-newAlpha),
                        alpha: newAlpha
                    )
                }
                currentPixel++
            }
        }
        
        let outputCGImage = CGBitmapContextCreateImage(context)
        let outputImage = UIImage(CGImage: outputCGImage!)
        
        // test write to file
        let documentsURL = NSFileManager.defaultManager().URLsForDirectory(.DocumentDirectory, inDomains: .UserDomainMask)[0]
        let fileURL = documentsURL.URLByAppendingPathComponent("filename2.png")
        UIImagePNGRepresentation(outputImage)!.writeToURL(fileURL, atomically: true)
        
        return outputImage
    }
    
    private func isEqual(pixel: UInt32, color: RGBA)->Bool{
        return alpha(pixel) > 0
        /*return (
            red(pixel) == color.red &&
            green(pixel) == color.green &&
            blue(pixel) == color.blue /*&&
            alpha(pixel) == color.alpha*/
        )*/
    }
    
    private func getColorComponents(color: UIColor)->RGBA{
        var r:CGFloat = 0
        var g:CGFloat = 0
        var b:CGFloat = 0
        var a:CGFloat = 0
        
        color.getRed(&r, green: &g, blue: &b, alpha: &a)
        
        return RGBA(red: (UInt8)(r*255), green: (UInt8)(g*255), blue: (UInt8)(b*255), alpha: (UInt8)(a*255))
    }

    private func red(color: UInt32) -> UInt8 {
        return UInt8(color & 255)
    }

    private func green(color: UInt32) -> UInt8 {
        return UInt8((color >> 8) & 255)
    }

    private func blue(color: UInt32) -> UInt8 {
        return UInt8((color >> 16) & 255)
    }

    private func alpha(color: UInt32) -> UInt8 {
        return UInt8((color >> 24) & 255)
    }

    private func getColorBytes(red red: UInt8, green: UInt8, blue: UInt8, alpha: UInt8) -> UInt32 {
        return UInt32(red) | (UInt32(green) << 8) | (UInt32(blue) << 16) | (UInt32(alpha) << 24)
    }
}