//
//  Layer.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 10/7/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import Foundation
import CoreData

@objc(Layer)
class Layer: Model, ModelProtocol{
    
    override class func getEntityName() -> String {
        return "Layer"
    }
    override func getEntityName() -> String {
        return "Layer"
    }
    
    @NSManaged var created: NSDate
    @NSManaged var descriptionString: String
    @NSManaged var hashString: String
    @NSManaged var hidden: Bool
    @NSManaged var kind: String
    @NSManaged var lastEdit: NSDate
    @NSManaged var name: String
    @NSManaged var localName: String
    @NSManaged var owner: String
    @NSManaged var publicBool: Bool
    @NSManaged var scheme: String
    @NSManaged var title: String
    @NSManaged var mergeFlag: Int64
    
    @NSManaged var projects: NSSet
    @NSManaged var attributes: NSSet
    @NSManaged var features: NSSet
    
    @NSManaged var notDeletedFeatures: NSSet
    
    /// Dynamic property for counting all not deleted features in this layer.
    var featuresCount: Int{
        get {
            let all = features.allObjects
            var count = 0
            for item in all{
                let feature = item as! Feature
                if(feature.hidden == false){
                    count++
                }
            }
            return count
        }
    }
    
    /**
     Returns path for feature pictures contained in this layer.
     
     - returns: Path as String, relative to the application documents directory.
     */
    var featuresPath: String{
        return FeaturePicture.PICTURES_PATH + "/" + localName + "/"
    }
    
    /**
     After deleting Layer delete folder containing feature pictures too.
     */
    override func didSave() {
        if(self.deleted){
            let fileManager = FileManager()
            fileManager.delete(featuresPath)
        }
        super.didSave()
    }
    
    /**
     Sets a local identifier for new object.
     */
    override func awakeFromInsert() {
        let dateFormatter = NSDateFormatter()
        dateFormatter.dateFormat = "yyyyMMddA"
        localName = dateFormatter.stringFromDate(NSDate())
        super.awakeFromInsert()
    }
    
    /**
     Updates hidden property of all related LIPs. We need to hide LayerInProject if Layer is going to be deleted.
     */
    override func didChangeValueForKey(key: String) {
        if(key == "hidden"){
            for item in projects{
                let lip = item as! LayerInProject
                lip.hidden = hidden
            }
        }
    }
}
