//
//  AttributeDetailController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 10/2/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import Foundation
import UIKit


protocol LayerAttributeDetailControllerDelegate{
    func didSaveAttribute()
}

class LayerAttributeDetailController: BaseTableViewController, DefaultFormValidatorDelegate{
    
    var delegate: LayerAttributeDetailControllerDelegate? = nil
    var validator: DefaultFormValidator? = nil
    var tempAttribute: TempLayerAttribute? = nil
    var tempLayerAttributesManager: TempLayerAttributesManager? = nil
    var layer: Layer? = nil
    
    @IBOutlet weak var nameField: UITextField!
    @IBOutlet weak var typeSegmentedControl: UISegmentedControl!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        initTypeSelect()
        initTitle()
        initValidation()
    }
    
    func prepareViewController(){
    }
    
    @IBAction func saveClick(sender: AnyObject) {
        validator?.validate(self)
    }
    
    @IBAction func cancelClick(sender: AnyObject) {
        dismissView()
    }
    
    private func initTypeSelect(){
        typeSegmentedControl.enabled = (tempAttribute == nil)
    }
    
    private func initTitle(){
        if(tempAttribute == nil){
            setTitleText(l("Add attribute"))
        }else{
            setTitleText(l("Edit attribute"))
            loadValues(tempAttribute!)
        }
    }
    
    private func initValidation(){
        validator = DefaultFormValidator()
        validator!.requiredField(nameField)
    }

    private func loadValues(item: TempLayerAttribute){
        nameField.text=item.title
        if(item.kind == AttributeKinds.number){
            typeSegmentedControl.selectedSegmentIndex = 0
        }else{
            typeSegmentedControl.selectedSegmentIndex = 1
        }
    }
    
    func validationSuccessful() {
        save()
        delegate?.didSaveAttribute()
        dismissView()
    }
    
    private func getKindValue()->String{
        if(typeSegmentedControl.selectedSegmentIndex == 0){
            return AttributeKinds.number
        }else{
            return AttributeKinds.string
        }
    }
    
    private func save(){
        if(checkForm()){
            if(tempLayerAttributesManager != nil){
                // unique check if title is changed. Checks both storage and temp attributes.
                if((tempAttribute?.title != nameField.text) && (tempLayerAttributesManager!.getByTitle(nameField.text!) != nil || tempLayerAttributesManager!.storage.getByLayerAndTitle(layer!, title: nameField.text!) != nil)){
                    alert(l("The entered name must be unique!"))
                }else{
                    if(tempAttribute == nil){
                        // add new
                        tempAttribute = TempLayerAttribute(title: nameField.text!.trim, kind: getKindValue())
                        tempLayerAttributesManager!.create(tempAttribute!)
                    }else{
                        // edit existing
                        tempAttribute!.title = nameField.text!.trim
                        tempAttribute!.kind = getKindValue()
                    }
                }
            }else{
                alert("Saving error!")
            }
        }else{
            alert("Please fill in all required fields!")
        }
    }
    
    private func checkForm()->Bool{
        nameField.text = nameField.text!.trim
        if(nameField.text! == ""){
            return false
        }
        return true
    }
    
}
