//
//  LayerAttributeStorage.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 1/11/16.
//  Copyright (c) 2016 MENDELU. All rights reserved.
//

import Foundation
import CoreData
import UIKit
import SwiftyJSON

class LayerAttributeStorage: CentralStorage{
    
    let ENTITY_NAME = LayerAttribute.getEntityName()
    
    required init(moc: NSManagedObjectContext, delegate: NSFetchedResultsControllerDelegate? = nil) {
        super.init(moc: moc, delegate: delegate, entityName: ENTITY_NAME)
    }
    
    override func getCreateURL(item: NSManagedObject) -> String {
        let layerAttribute = item as! LayerAttribute
        let layer = layerAttribute.layer
        return "/layer/"+layer.scheme+"/"+layer.name+"/attribute"
    }
    
    override func getItemURL(item: NSManagedObject) -> String {
        let object = item as! LayerAttribute
        return "/layer/"+object.layer.scheme+"/"+object.layer.name+"/attribute/"+object.name
    }
    
    override func getItemByURLParts(urlParts: [String]) -> NSManagedObject? {
        if((urlParts.count)==6){
            return getByLayerAndName(urlParts[3], attrName: urlParts[5])
        }
        return nil
    }
    
    override func getMapping() -> NSDictionary? {
        return [
            "name": "name", 
            "hash": "hashString",
            "kind": "kind",
            "title": "title"
        ]
    }
    
    override func getRelationsURLMapping() -> NSDictionary? {
        return [
            3: "Layer"
        ]
    }
    
    override func newEntity() -> NSManagedObject {
        let item = super.newEntity() as! LayerAttribute
        return item
    }
    
    override func getDefaultSort() -> [NSSortDescriptor] {
        return [NSSortDescriptor(key: "title", ascending: true)]
    }
    
    func getAllForLayer(layer: Layer)->NSFetchedResultsController{
        let predicate = NSPredicate(format: "("+getDefaultPredicate()+") AND (layer = %@)", layer)
        return local.getAll(entityName, predicate: predicate, sort: getDefaultSort(), delegate: delegate)
    }
    
    func getByLayerAndName(layerName: String, attrName: String)->LayerAttribute?{
        let predicate = NSPredicate(format: "(name = %@) AND (layer.name=%@)", attrName, layerName)
        let items = local.getAll(entityName, predicate: predicate, sort: getDefaultSort(), delegate: nil)
        if(items.fetchedObjects?.count > 0){
            return items.fetchedObjects?.first as! LayerAttribute?
        }
        return nil
    }
    
    func getByLayerAndTitle(layer: Layer, title: String)->LayerAttribute?{
        let predicate = NSPredicate(format: "(title = %@) AND (layer=%@)", title, layer)
        let items = local.getAll(entityName, predicate: predicate, sort: getDefaultSort(), delegate: nil)
        if(items.fetchedObjects?.count > 0){
            return items.fetchedObjects?.first as! LayerAttribute?
        }
        return nil
    }
}
