//
//  LayerAttributesController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 14/01/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import UIKit
import CoreData

protocol LayerAttributesControllerDelegate{
    func didChangeAttributes()
}

class LayerAttributesController: DatabaseTableViewController, LayerAttributeDetailControllerDelegate {
    
    var layer: Layer? = nil
    var layerTitle = ""
    var addItemImmediatly = false
    var tempLayerAttributesManager: TempLayerAttributesManager? = nil
    var delegate: LayerAttributesControllerDelegate? = nil
    
    @IBOutlet weak var addAttributeBottomButton: UIButton!
    
    var isOwnerOfLayer = false
    
    override func viewDidLoad() {
        super.viewDidLoad()
        configureView()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        addAttributeBottomButton.hidden = !isIPad()
        reloadData()
        checkOwner()
        if(addItemImmediatly){
            addItemImmediatly = false
            addItem()
        }
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    // MARK: - Local functions
    
    private func configureView(){
        if(!isIPad()){
            self.navigationItem.leftBarButtonItem = UIBarButtonItem(title: l("Done"), style: UIBarButtonItemStyle.Done, target: self, action: "doneClick")
            setTitleText(String(format: l("Layer %@"), arguments: [layerTitle]))
        }
    }
    
    func checkOwner(){
        if(layer != nil && layer?.owner != Settings.username){
            addAttributeBottomButton.enabled = false
            isOwnerOfLayer = false
        }else{
            isOwnerOfLayer = true
        }
    }
    
    func doneClick(){
        dismissView()
    }
    
    func addItem(){
        performSegueWithIdentifier("add-attribute-nav", sender: self)
    }
    
    func editItem(indexPath: NSIndexPath?){
        let cell = tableView(tableView, cellForRowAtIndexPath: indexPath!)
        
        let navCtrl = self.storyboard?.instantiateViewControllerWithIdentifier("attribute-detail-nav") as! UINavigationController
        let target = navCtrl.childViewControllers[0] as! LayerAttributeDetailController
        let object = tempLayerAttributesManager?.attributes[indexPath!.row]
        target.tempAttribute = object
        target.tempLayerAttributesManager = tempLayerAttributesManager
        target.layer = layer
        target.delegate = self
        
        // align the dialog to the right of the row
        tableView.setEditing(false, animated: true)
        navCtrl.popoverPresentationController?.sourceView = cell as UIView
        navCtrl.popoverPresentationController?.sourceRect = cell.bounds
        navCtrl.popoverPresentationController?.permittedArrowDirections = UIPopoverArrowDirection.Left
        
        presentViewController(navCtrl, animated: true, completion: nil)
    }
    
    func deleteItem(indexPath: NSIndexPath?) {
        if(indexPath != nil){
            let item = tempLayerAttributesManager?.attributes[indexPath!.row]
            if(item != nil){
                let optionMenu = UIAlertController(title: String(format: l("Really delete attribute %@?"), item!.title), message: l("All relevant feature values will be deleted too."), preferredStyle: .ActionSheet)
                
                let action = UIAlertAction(title: "Delete", style: .Destructive, handler: {
                    (alert: UIAlertAction!) -> Void in
                    self.tempLayerAttributesManager!.delete(indexPath!.row)
                    self.delegate?.didChangeAttributes()
                    self.reloadData()
                })
                
                let cancelAction = UIAlertAction(title: "Cancel", style: .Default, handler: {
                    (alert: UIAlertAction!) -> Void in
                })
                
                optionMenu.addAction(action)
                optionMenu.addAction(cancelAction)
                
                tableView.setEditing(false, animated: true)
                let cell = tableView(tableView, cellForRowAtIndexPath: indexPath!)
                optionMenu.popoverPresentationController?.sourceView = cell
                optionMenu.popoverPresentationController?.sourceRect = cell.bounds
                optionMenu.popoverPresentationController?.permittedArrowDirections = UIPopoverArrowDirection.Right
                
                presentViewController(optionMenu, animated: true, completion: nil)
            }
        }
    }
    
    // MARK: - Segues
    override func shouldPerformSegueWithIdentifier(identifier: String, sender: AnyObject?) -> Bool {
        if(self.editing){
            return false
        }else{
            return super.shouldPerformSegueWithIdentifier(identifier, sender: sender)
        }
    }
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if segue.identifier == "add-attribute-nav" {
            let navi = segue.destinationViewController as! UINavigationController
            let target = navi.viewControllers[0] as! LayerAttributeDetailController
            target.tempLayerAttributesManager = tempLayerAttributesManager
            target.layer = layer
            target.delegate = self
            target.prepareViewController()
        }
    }
    
    // MARK: - TABLEVIEW 
    func didSaveAttribute() {
        reloadData()
        self.delegate?.didChangeAttributes()
    }
    
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    override func tableView(tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        return l("Attributes")
    }
    
    override func getAll() -> NSFetchedResultsController? {
        return nil
    }
    
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if(tempLayerAttributesManager == nil){
            return 0
        }else{
            return tempLayerAttributesManager!.attributes.count
        }
    }
    
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("Cell", forIndexPath: indexPath) as UITableViewCell
        self.configureCell(cell, atIndexPath: indexPath)
        return cell
    }
    
    override func configureCell(cell: UITableViewCell, atIndexPath indexPath: NSIndexPath) {
        let object = tempLayerAttributesManager!.attributes[indexPath.row]
        cell.textLabel!.text = object.title
        cell.detailTextLabel!.text = object.kind
    }
    
    override func tableView(tableView: UITableView, editActionsForRowAtIndexPath indexPath: NSIndexPath) -> [UITableViewRowAction]?  {
        let delete = UITableViewRowAction(style: UITableViewRowActionStyle.Default, title: "Delete" , handler: { (action:UITableViewRowAction, indexPath:NSIndexPath) -> Void in
            self.deleteItem(indexPath)
        })
        delete.backgroundColor = SwipeActionsColors.delete
        
        let edit = UITableViewRowAction(style: UITableViewRowActionStyle.Default, title: "Edit" , handler: { (action:UITableViewRowAction, indexPath:NSIndexPath) -> Void in
            self.editItem(indexPath)
        })
        edit.backgroundColor = SwipeActionsColors.edit
        
        return [delete, edit]
    }
    
    override func tableView(tableView: UITableView, canEditRowAtIndexPath indexPath: NSIndexPath) -> Bool {
        return isOwnerOfLayer
    }
}


