//
//  LayerDetailController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 10/1/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import Foundation
import UIKit

protocol LayerDeatilControllerDelegate{
    func didSaveLayer()
    func didCancelLayer()
}

class LayerDetailController: BaseTableViewController, DefaultFormValidatorDelegate, SelectSymbolControllerDelegate, SelectColorControllerDelegate, LayerAttributesControllerDelegate{
    
    @IBOutlet weak var typeSegmentedControl: UISegmentedControl!
    @IBOutlet weak var nameField: UITextField!
    @IBOutlet weak var descriptionField: UITextField!
    @IBOutlet weak var zoomLabel: UILabel!
    @IBOutlet weak var zoomSlider: UISlider!
    @IBOutlet weak var publicSwitch: UISwitch!
    @IBOutlet weak var symbolCell: UITableViewCell!
    @IBOutlet weak var symbolIcon: UIImageView!
    @IBOutlet weak var colorView: UILabel!
    @IBOutlet weak var transparencyCell: UITableViewCell!
    @IBOutlet weak var transparencyLabel: UILabel!
    @IBOutlet weak var transparencySlider: UISlider!
    
    @IBOutlet weak var attributesText: UILabel!
    @IBOutlet weak var editAttributesButton: UIButton!
    @IBOutlet weak var addAttributeButton: UIButton!
    
    var delegate: LayerDeatilControllerDelegate? = nil
    
    var layer: Layer? = nil
    var project: Project? = nil
    var layerInProject: LayerInProject? = nil
    
    var storage: LayerStorage? = nil
    /// If layerInProjectStorage is nil, only Layer properties are saved and Appearance section is hidden.
    var layerInProjectStorage: LayerInProjectStorage? = nil
    var layerAttributeStorage: LayerAttributeStorage? = nil
    
    var validator: DefaultFormValidator? = nil
    
    var selectedColor: String = config.markers.colors[0]
    var selectedSymbol: String? = config.markers.symbols[0]
    var tempLayerAttributesManager: TempLayerAttributesManager? = nil
    
    override func viewDidLoad() {
        super.viewDidLoad()
        configureView()
    }
    
    private func configureView(){
        if(layer == nil){
            setTitleText(l("Add layer"))
            let temp = LayerStorage(moc: storage!.managedObjectContext)
            nameField.placeholder = temp.getUniqueTitle()
        }else{
            setTitleText(l("Edit layer"))
            typeSegmentedControl.enabled = false
            loadValues(layer!)
            checkOwner()
        }
        
        attributesText.hidden = false
        editAttributesButton.hidden = false
        addAttributeButton.hidden = true
        
        navigationController?.navigationBarHidden = isIPad()
        automaticallyAdjustsScrollViewInsets = !isIPad()
        
        updateAppearanceCells()
        updateAttributesCell()
        initValidation()
    }
    
    private func initValidation(){
        validator = DefaultFormValidator()
        validator!.requiredField(nameField)
    }
    
    private func loadValues(item: Layer){
        // properties
        if(item.kind==LayerTypes.point.lowercaseString){
            typeSegmentedControl.selectedSegmentIndex=0
        }else if(item.kind==LayerTypes.line.lowercaseString){
            typeSegmentedControl.selectedSegmentIndex=1
        }else if(item.kind==LayerTypes.polygon.lowercaseString){
            typeSegmentedControl.selectedSegmentIndex=2
        }
        
        nameField.text=item.title
        descriptionField.text=item.descriptionString
        publicSwitch.on = item.publicBool
        
        // appearance
        if(layerInProject != nil){
            zoomSlider.value = layerInProject!.zoomThreshold as Float
            updateZoomLabel()
            if(layerInProject!.color != nil){
                selectedColor = layerInProject!.color!
            }
            colorView.backgroundColor = UIColor(hexString: selectedColor)
            if(layerInProject?.symbology != nil){
                selectedSymbol = layerInProject!.symbology
                symbolIcon.image = Symbols.getImage(layerInProject!.symbology, color: UIColor(hexString: selectedColor))
            }
            transparencySlider.value = layerInProject!.transparency as Float
            updateTransparencyLabel()
        }
        
        // attributes
        tempLayerAttributesManager!.load(layer!)
    }
    
    func checkOwner(){
        if(layer != nil && layer?.owner != Settings.username){
            nameField.enabled = false
            descriptionField.enabled = false
            publicSwitch.enabled = false
            addAttributeButton.enabled = false
            editAttributesButton.enabled = false
            storage = nil
        }
    }
    
    private func checkForm()->Bool{
        nameField.text = nameField.text!.trim
        if(nameField.text! == ""){
            return false
        }
        return true
    }
    
    func validationSuccessful() {
        saveLayer()
        defaultDelegate?.didChangeData()
        delegate?.didSaveLayer()
        dismissView()
    }
    
    private func saveLayer(){
        
        // if adding layer
        var newItem = false
        if(layer == nil){
            newItem = true
        }
        
        // check required fields
        if(!checkForm()){
            alert("Please fill in all required fields!")
            
        // unique check if title was changed
        }else if(layer?.title != nameField.text! && storage!.getByTitle(nameField.text!) != nil){
            alert(l("The entered name must be unique!"))
        
        // delete check, check Layer and LIP too if LIP is edited in this controller too
        }else if(!newItem && (layer?.managedObjectContext == nil || (layerInProjectStorage != nil && layerInProject?.managedObjectContext == nil))){
            alert(l("Layer cannot be saved. Item has already been deleted!"))
            self.dismissViewControllerAnimated(true, completion: nil)
        
        // everything ok
        }else{
            
            // add new
            if(newItem){
                layer = storage!.newEntity() as? Layer
                layerInProject = layerInProjectStorage?.newEntity() as? LayerInProject
                layerInProject?.order = project!.layers.count + 1
            }
            
            // set up values
            let selectedType = typeSegmentedControl.selectedSegmentIndex
            if(selectedType == 0){
                layer!.kind = LayerTypes.point.lowercaseString
            }else if(selectedType == 1){
                layer!.kind = LayerTypes.line.lowercaseString
            }else if(selectedType == 2){
                layer!.kind = LayerTypes.polygon.lowercaseString
            }
            
            layer!.title = nameField.text!
            layer!.descriptionString = descriptionField.text!.trim
            layer!.publicBool = publicSwitch.on
            
            layerInProject?.color = selectedColor
            layerInProject?.project = project!
            layerInProject?.layer = layer!
            layerInProject?.symbology = selectedSymbol!
            layerInProject?.transparency = transparencySlider.value
            layerInProject?.zoomThreshold = zoomSlider.value
            layerInProject?.hidden = false
            
            // save
            if(newItem){
                storage?.create(layer!)
                layerInProjectStorage?.create(layerInProject!)
            }else{
                storage?.update(layer!)
                layerInProjectStorage?.update(layerInProject!)
            }
            
            // save changes in layer attributes
            tempLayerAttributesManager!.saveAttributesPernamentlyToLayer(layer!)
            
            // start sync with the server
            if(storage != nil){
                storage!.synchronise()
            }else{
                layerInProjectStorage?.synchronise()
            }
        }
    }
    
    @IBAction func saveClick(sender: AnyObject) {
        // Supply empty title field by placeholder with unique layer name (only for adding new layer).
        if(nameField.text == ""){
            nameField.text = nameField.placeholder
        }
        validator?.validate(self)
    }
    
    @IBAction func cancelClick(sender: AnyObject) {
        delegate?.didCancelLayer()
        dismissView()
    }
    
    @IBAction func typeChanged(sender: AnyObject) {
        updateAppearanceCells()
    }
    
    func updateAppearanceCells(){
        if(typeSegmentedControl.selectedSegmentIndex==0){
            symbolCell.hidden = false
            transparencyCell.hidden = true
        }else{
            symbolCell.hidden = true
            transparencyCell.hidden = false
        }
        
        colorView.backgroundColor=UIColor(hexString: selectedColor)
        
        // require recalculation of cell heights
        tableView.beginUpdates()
        tableView.endUpdates()
    }
    
    func updateAttributesCell(){
        if(tempLayerAttributesManager!.attributes.count > 0){
            addAttributeButton.hidden = true
            editAttributesButton.hidden = false
            attributesText.hidden = false
            attributesText.text = tempLayerAttributesManager?.getAllAsString()
        }else{
            addAttributeButton.hidden = false
            editAttributesButton.hidden = true
            attributesText.hidden = true
            attributesText.text = ""
        }
    }
    
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        if(isIPad()){
            if(layerInProjectStorage == nil){
                return 1 // only Properties section
            }else{
                return 2 // Properties and Appearance
            }
        }else{
            return 3 // we cannot hide the middle section, display all on iphones
        }
    }
    
    override func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        
        // hide Not available cell if editing LayerInProject (not only Layer) on iPhone.
        if(layerInProject != nil && indexPath.section == 1 && indexPath.row == 4){
            return 0
        }
        
        // hide transparency choice for point layer type
        if(typeSegmentedControl.selectedSegmentIndex==0 && indexPath.section==1 && indexPath.row==3){
            return 0
        }
        
        // hide symbol choice for line and polygon layer types
        if(typeSegmentedControl.selectedSegmentIndex>0 && indexPath.section==1 && indexPath.row==1){
            return 0
        }
        
        // otherwise use default height from the storyboard
        return super.tableView(tableView, heightForRowAtIndexPath: indexPath)
    }
    
    @IBAction func zoomOutClick(sender: AnyObject) {
        if(zoomSlider.value != zoomSlider.minimumValue){
            zoomSlider.value-=1
        }
        updateZoomLabel()
    }
    
    @IBAction func zoomInClick(sender: AnyObject) {
        if(zoomSlider.value != zoomSlider.maximumValue){
            zoomSlider.value+=1
        }
        updateZoomLabel()
    }
    
    @IBAction func zoomChanged(sender: UISlider) {
        updateZoomLabel()
    }
    
    private func updateZoomLabel(){
        zoomLabel.text = String(format: "%.f", round(zoomSlider.value))
    }
    
    @IBAction func decreaseTransparencyClick(sender: AnyObject) {
        if(transparencySlider.value != transparencySlider.minimumValue){
            transparencySlider.value-=25.5
        }
        updateTransparencyLabel()
    }
    
    @IBAction func increaseTransparencyClick(sender: AnyObject) {
        if(transparencySlider.value != transparencySlider.maximumValue){
            transparencySlider.value+=25.5
        }
        updateTransparencyLabel()
    }
    
    @IBAction func transparencyChanged(sender: UISlider) {
        updateTransparencyLabel()
    }
    
    private func updateTransparencyLabel(){
        let percentage = (transparencySlider.value*100)/255
        transparencyLabel.text = String(format: "%.f", round(percentage)) + "%"
    }
    
    func didSelectSymbol(name: String) {
        selectedSymbol = name
        symbolIcon.image = Symbols.getImage(name, color: UIColor(hexString: selectedColor))
    }
    
    func didSelectColor(color: String) {
        selectedColor = color
        colorView.backgroundColor = UIColor(hexString: color)
        didSelectSymbol(selectedSymbol!)
    }
    
    func didChangeAttributes() {
        updateAttributesCell()
    }
    
    // MARK: - SEGUES
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        if(indexPath.section == 2 && indexPath.row==0 && tempLayerAttributesManager?.attributes.count > 0){
            performSegueWithIdentifier("edit-attributes", sender: self)
        }
    }
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if(segue.identifier=="select-symbol"){
            let navi = segue.destinationViewController as! UINavigationController
            let target = navi.viewControllers[0] as! SelectSymbolController
            target.delegate = self
        }else if(segue.identifier=="select-color"){
            let navi = segue.destinationViewController as! UINavigationController
            let target = navi.viewControllers[0] as! SelectColorController
            target.delegate = self
        }else if(segue.identifier=="add-attribute" || segue.identifier=="edit-attributes"){
            NSLog("segue")
            let navi = segue.destinationViewController as! UINavigationController
            let target = navi.viewControllers[0] as! LayerAttributesController
            target.tempLayerAttributesManager = tempLayerAttributesManager
            target.storage = layerAttributeStorage
            target.delegate = self
            target.layer = layer
            target.layerTitle = nameField.text!
            if(segue.identifier=="add-attribute"){
                target.addItemImmediatly = true
            }
        }
    }
}
