//
//  LayerExporter.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 02/05/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import Foundation
import SwiftyJSON

/** Exports layer properties, layer attributes, features and features attributes to a JSON file in the app Documents directory. */
class LayerJSONExporter{
    var layer: Layer
    var mapping: DataMapping?
    
    init(layer: Layer, localStorage: LocalStorage){
        self.layer = layer
        self.mapping = DataMapping(localStorage: localStorage)
    }
    
    func run(){
        let features = exportFeatures()
        let layerProperties = exportLayerProperties()
        let json = [
            "type":"FeatureCollection",
            "properties":layerProperties,
            "attributes":exportLayerAttributes(),
            "features": features
        ]
        saveToFile(makeJSONString(json), fileName: getFileName())
    }
    
    private func getFileName()->String{
        return String(format: "gisella-export-%@.json", sanitizeFileName(layer.title))
    }
    
    private func exportLayerProperties()->NSMutableDictionary{
        let storage = LayerStorage(moc: mapping!.localStorage!.managedObjectContext)
        let output = mapping?.getRequestRawJSON(layer, mapping: storage.getOutputMapping()!)
        return output!["properties"] as! NSMutableDictionary
    }
    
    private func exportFeatures()->[NSMutableDictionary]{
        let features = layer.features
        let featureStorage = FeatureStorage(moc: mapping!.localStorage!.managedObjectContext)
        var featuresJSON = [NSMutableDictionary]()
        for feature in features{
            let featureJSON = mapping?.getRequestRawJSON(feature as! Model, mapping: featureStorage.getOutputMapping()!)
            featureJSON!["attributes"] = exportFeatureAttributes(feature as! Feature)
            featuresJSON.append(featureJSON!)
        }
        return featuresJSON
    }
    
    private func exportFeatureAttributes(feature: Feature)->[NSMutableDictionary]{
        let attrs = feature.attributes
        let attrsStorage = FeatureAttributeStorage(moc: mapping!.localStorage!.managedObjectContext)
        var attrsJSON = [NSMutableDictionary]()
        for attr in attrs{
            let attrJSON = mapping?.getRequestRawJSON(attr as! Model, mapping: attrsStorage.getOutputMapping()!)
            attrsJSON.append(attrJSON!)
        }
        return attrsJSON
    }
    
    private func exportLayerAttributes()->[NSMutableDictionary]{
        let attrs = layer.attributes
        let attrsStorage = LayerAttributeStorage(moc: mapping!.localStorage!.managedObjectContext)
        var attrsJSON = [NSMutableDictionary]()
        for attr in attrs{
            let attrJSON = mapping?.getRequestRawJSON(attr as! Model, mapping: attrsStorage.getOutputMapping()!)
            attrsJSON.append(attrJSON!)
        }
        return attrsJSON
    }
    
    private func saveToFile(text: String, fileName: String){
        if let dir = NSSearchPathForDirectoriesInDomains(NSSearchPathDirectory.DocumentDirectory, NSSearchPathDomainMask.AllDomainsMask, true).first {
            let path = NSURL(fileURLWithPath: dir).URLByAppendingPathComponent(fileName)
            
            do {
                try text.writeToURL(path, atomically: false, encoding: NSUTF8StringEncoding)
            }
            catch {
                /* error handling here */
            }
        }
    }

}