//
//  LayerInProjectStorage.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 9/15/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import Foundation
import CoreData
import UIKit
import SwiftyJSON

class LayerInProjectStorage: CentralStorage{
    
    let ENTITY_NAME = LayerInProject.getEntityName()
    
    required init(moc: NSManagedObjectContext, delegate: NSFetchedResultsControllerDelegate? = nil) {
        super.init(moc: moc, delegate: delegate, entityName: ENTITY_NAME)
    }
    
    override func getCreateURL(item: NSManagedObject) -> String {
        let layerInProject = item as! LayerInProject
        let project = layerInProject.project!
        let layer = layerInProject.layer!
        return "/project/"+project.scheme+"/"+project.name+"/layer/"+layer.scheme+"/"+layer.name
    }
    
    override func getItemURL(item: NSManagedObject) -> String {
        let object = item as! LayerInProject
        return "/project/"+object.project!.scheme+"/"+object.project!.name+"/layer/"+object.layer!.scheme+"/"+object.layer!.name
    }
    
    override func getItemByURLParts(urlParts: [String]) -> NSManagedObject? {
        if((urlParts.count)==7){
            return getByProjectAndLayer(urlParts[3], layerName: urlParts[6])
        }
        return nil
    }
    
    override func getMapping() -> NSDictionary? {
        return [
            "hash": "hashString",
            "properties": [
                "color": "color",
                "transparency": "transparency",
                "zoom_threshold": "zoomThreshold",
                "visibility": "visibility",
                "order": "order",
                "symbology": "symbology"
            ]
        ]
    }
    
    override func getRelationsURLMapping() -> NSDictionary? {
        return [
            6: "Layer",
            3: "Project"
        ]
    }
    
    override func newEntity() -> NSManagedObject {
        let newMapLayer = super.newEntity() as! LayerInProject
        newMapLayer.hidden = false
        newMapLayer.visibility = true
        return newMapLayer
    }
    
    override func getDefaultSort() -> [NSSortDescriptor] {
        return [NSSortDescriptor(key: "order", ascending: true)]
    }
    
    func getAllForProject(project: Project)->NSFetchedResultsController{
        let predicate = NSPredicate(format: "("+getDefaultPredicate()+") AND (project = %@)", project)
        return local.getAll(entityName, predicate: predicate, sort: getDefaultSort(), delegate: delegate)
    }
    
    func getByProjectAndLayer(projectName: String, layerName: String)->LayerInProject?{
        let predicate = NSPredicate(format: "(project.name = %@) AND (layer.name=%@)", projectName, layerName)
        let items = local.getAll(entityName, predicate: predicate, sort: getDefaultSort(), delegate: nil)
        if(items.fetchedObjects?.count > 0){
            return items.fetchedObjects?.first as! LayerInProject?
        }
        return nil
    }
    
    override func getByName(name: String?) -> Layer? {
        // cannot get layer in project by name
        return nil
    }
}
