//
//  LayerStorage.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 9/15/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import Foundation
import CoreData
import UIKit
import SwiftyJSON

class LayerStorage: CentralStorage{
    
    let ENTITY_NAME = Layer.getEntityName()
    
    required init(moc: NSManagedObjectContext, delegate: NSFetchedResultsControllerDelegate? = nil) {
        super.init(moc: moc, delegate: delegate, entityName: ENTITY_NAME)
    }
    
    override func getCreateURL(item: NSManagedObject) -> String {
        return "/layer"
    }
    
    override func getItemURL(item: NSManagedObject) -> String {
        let layer = item as! Layer
        return "/layer/"+layer.scheme+"/"+layer.name
    }
    
    override func getItemByURLParts(urlParts: [String]) -> NSManagedObject? {
        if((urlParts.count)==4){
            let name = urlParts[3]
            return getByName(name)
        }
        return nil
    }

    override func getMapping() -> NSDictionary? {
        return [
            "name": "name",
            "scheme": "scheme",
            "owner": "owner",
            "hash": "hashString",
            "properties": [
                "title": "title",
                "public": "publicBool",
                "description": "descriptionString",
                "kind": "kind",
                "created": "created",
                "last_edit": "lastEdit"
            ]
        ]
    }
    
    override func newEntity() -> NSManagedObject {
        let newLayer = super.newEntity() as! Layer
        // default values
        newLayer.created = NSDate()
        newLayer.lastEdit = NSDate()
        newLayer.hidden = false
        return newLayer
    }
    
    override func getDefaultSort() -> [NSSortDescriptor] {
        return [NSSortDescriptor(key: "title", ascending: true)]
    }
    
    override func delete(item: NSManagedObject) {
        let layer = item as! Layer
        if(layer.owner != Settings.username){
            alert("Layer \(layer.title) cannot be deleted. You are not its owner.")
        }else{
            super.delete(item)
        }
    }
    
    /**
     Returns all layers that are not pinned to the given project (LayerInProject object between the layer and the given project does not exist).
     
     - parameter project: The searched project
     
     - returns: Layers
     */
    func getNotPinnedToProject(project: Project)->NSFetchedResultsController{
        let predicate = NSPredicate(format: "("+getDefaultPredicate()+") AND SUBQUERY(projects, $lip, $lip.project = %@).@count = 0", project)
        return local.getAll(entityName, predicate: predicate, sort: getDefaultSort(), delegate: delegate)
    }
    
    /**
     Finds a not used title for a new layer. The naming pattern is LayerX where X is an integer starting at layers count.
     
     - returns: The unique title
     */
    func getUniqueTitle()->String{
        var title = ""
        if var counter = getAll().fetchedObjects?.count {
            repeat {
                title = l("Layer\(counter)")
                counter++
            } while(getByTitle(title) != nil)
            return title
        }
        return ""
    }
}
