//
//  LayerController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 19/02/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import Foundation
import UIKit
import CoreData

class LayersController: DatabaseTableViewController, LayerDeatilControllerDelegate, MapControllerDelegate {
    
    var layerAttributeStorage: LayerAttributeStorage? = nil
    var delegate: BaseTableViewControllerDelegate? = nil
    
    var enabledLayers: [Bool] = []
    
    var editingToolbarButtons: [UIBarButtonItem] = []
    
    var addNavBarButton: UIBarButtonItem? = nil
    
    @IBOutlet weak var editNavBarButton: UIBarButtonItem!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        storage = LayerStorage(moc: managedObjectContext!, delegate: self)
        layerAttributeStorage = LayerAttributeStorage(moc: managedObjectContext!, delegate: nil)
        configureView()
        updateNotification!.registerForUpdates(["Layer"])
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        navigationController?.setToolbarHidden(true, animated: false)
        storage?.delegate=self
    }
    
    override func viewWillDisappear(animated: Bool) {
        super.viewWillDisappear(animated)
        setEditing(false, animated: false)
    }
    
    func configureView(){
        addNavBarButton = UIBarButtonItem(barButtonSystemItem: UIBarButtonSystemItem.Add, target: self, action: "addLayerClick:")
        self.navigationItem.rightBarButtonItem = addNavBarButton!
        //self.navigationItem.leftBarButtonItem = editButtonItem()
        initToolbarButtons()
        updateToolbarState()
    }
    
    func didSaveLayer() {
        reloadData()
        delegate?.didChangeData()
    }
    
    func didCancelLayer() {
        
    }
    
    private func setTypeIcon(icon: UIImageView, type: String){
        switch type{
        case LayerTypes.point.lowercaseString:
            icon.image = UIImage(named: Images.LayerPoint)
        case LayerTypes.line.lowercaseString:
            icon.image = UIImage(named: Images.LayerLine)
        case LayerTypes.polygon.lowercaseString:
            icon.image = UIImage(named: Images.LayerPolygon)
        default:
            break
        }
    }
    
    @IBAction func switchEditingStateClick(sender: UIBarButtonItem) {
        hideTabbarBeforeMultipleEditing()
        setEditing(!editing, animated: true)
    }
    
    override func setEditing(editing: Bool, animated: Bool) {
        let last = self.editing
        
        super.setEditing(editing, animated: animated)
        
        if(last != editing){
            if(editing == true){
                editNavBarButton.title = l("Done")
                editNavBarButton.style = UIBarButtonItemStyle.Done
            }else{
                editNavBarButton.title = l("Edit")
                editNavBarButton.style = UIBarButtonItemStyle.Plain
                
                showTabbarAfterDoneEditing()
            }
        }
    }
    
    override func tableView(tableView: UITableView, didEndEditingRowAtIndexPath indexPath: NSIndexPath) {
        super.tableView(tableView, didEndEditingRowAtIndexPath: indexPath)
        showTabbarAfterDoneEditing()
    }
    
    private func hideTabbarBeforeMultipleEditing(){
        if(editing == false){
            // start multiple editing
            
            //self.editing = true
            //tableView!.allowsMultipleSelection = true
            
            
            
            // hide the tab bar and resize the main view to fill space after the hidden tab bar
            self.tabBarController?.tabBar.hidden = true
            var biggerFrame = tabBarController!.view.frame;
            biggerFrame.size.height += tabBarController!.tabBar.frame.size.height;
            tabBarController!.view.frame = biggerFrame ;
        }
    }
    
    private func showTabbarAfterDoneEditing(){
        if(editing == false && self.tabBarController?.tabBar.hidden == true){
            
            // end multiple editing
            
            //self.editing = false
            //tableView!.allowsMultipleSelection = false
            
            
            // show the tab bar and resize the main view to fit with the displayed tab bar
            self.tabBarController?.tabBar.hidden = false
            var biggerFrame = tabBarController!.view.frame;
            biggerFrame.size.height -= tabBarController!.tabBar.frame.size.height;
            tabBarController!.view.frame = biggerFrame ;
            
        }
    }
    
    
    func sharedIconClick(recognizer: UITapGestureRecognizer){
        alert(l("This layer is public."))
    }
    
    func typeIconClick(recognizer: UITapGestureRecognizer){
        let image = recognizer.view as! UIImageView
        let view = image.superview! as UIView
        let cell = view.superview!.superview as! LayerCell
        let type = cell.layerObject!.kind
        alert(String(format: l("This layer type is %@."), type))
    }
    
    func initToolbarButtons(){
        navigationController?.setToolbarHidden(true, animated: false)
        editingToolbarButtons = [
            UIBarButtonItem(title: l("Edit"), style: .Plain, target: self, action: "editSelectedClick"),
            UIBarButtonItem(barButtonSystemItem: UIBarButtonSystemItem.FlexibleSpace, target: nil, action: nil),
            UIBarButtonItem(title: l("Remove"), style: .Plain, target: self, action: "deleteSelected:"),
            UIBarButtonItem(barButtonSystemItem: UIBarButtonSystemItem.FlexibleSpace, target: nil, action: nil),
            UIBarButtonItem(title: l("More actions..."), style: .Plain, target: self, action: "showMoreActionsForSelected:")
        ]
    }
    
    override func updateToolbarState(){
        let count = tableView.indexPathsForSelectedRows?.count
        if(editing && count > 0){
            navigationController?.setToolbarHidden(false, animated: false)
            self.toolbarItems = editingToolbarButtons
            (self.toolbarItems![0] as UIBarButtonItem).enabled = (count==1) // edit button only for one selected item
            (self.toolbarItems![2] as UIBarButtonItem).enabled = (count>0)
            (self.toolbarItems![4] as UIBarButtonItem).enabled = (count>0)
        }else{
            self.toolbarItems = []
            navigationController?.setToolbarHidden(true, animated: false)
        }
    }
    
    func addLayerClick(sender: UIBarButtonItem){
        if(isIPad()){
            let navCtrl = self.storyboard?.instantiateViewControllerWithIdentifier("layer-attributes-nav") as! UINavigationController
            let target = navCtrl.childViewControllers[0] as! LayerDetailWithAttributtesController
            target.storage = storage as? LayerStorage
            target.layerAttributeStorage = layerAttributeStorage
            target.tempLayerAttributesManager = TempLayerAttributesManager(storage: layerAttributeStorage!)
            target.delegate = self
            presentViewController(navCtrl, animated: true, completion: nil)
        }else{
            let navCtrl = self.storyboard?.instantiateViewControllerWithIdentifier("layer-detail-nav") as! UINavigationController
            let target = navCtrl.childViewControllers[0] as! LayerDetailController
            target.storage = storage as? LayerStorage
            target.layerAttributeStorage = layerAttributeStorage
            target.tempLayerAttributesManager = TempLayerAttributesManager(storage: layerAttributeStorage!)
            target.delegate = self
            presentViewController(navCtrl, animated: true, completion: nil)
        }
    }
    
    func editSelectedClick() {
        if let indexPath = self.tableView.indexPathForSelectedRow {
            editItem(indexPath)
        }
    }
    
    func editItem(indexPath: NSIndexPath){
        if(isIPad()){
            let navCtrl = self.storyboard?.instantiateViewControllerWithIdentifier("layer-attributes-nav") as! UINavigationController
            let target = navCtrl.childViewControllers[0] as! LayerDetailWithAttributtesController
            target.storage = storage as? LayerStorage
            target.layerAttributeStorage = layerAttributeStorage
            let object = storage!.get(indexPath) as! Layer
            target.layer = object
            target.tempLayerAttributesManager = TempLayerAttributesManager(storage: layerAttributeStorage!)
            target.delegate = self
            presentViewController(navCtrl, animated: true, completion: nil)
        }else{
            let navCtrl = self.storyboard?.instantiateViewControllerWithIdentifier("layer-detail-nav") as! UINavigationController
            let target = navCtrl.childViewControllers[0] as! LayerDetailController
            target.storage = storage as? LayerStorage
            target.layerAttributeStorage = layerAttributeStorage
            let object = storage!.get(indexPath) as! Layer
            target.layer = object
            target.tempLayerAttributesManager = TempLayerAttributesManager(storage: layerAttributeStorage!)
            target.delegate = self
            presentViewController(navCtrl, animated: true, completion: nil)
        }
    }
    
    func exportItem(layer: Layer?, indexPath: NSIndexPath?) {
        if(layer != nil && layer?.managedObjectContext != nil && indexPath != nil){
            // show confirm dialog
            let optionMenu = UIAlertController(title: String(format: l("Choose export format for layer %@?"), layer!.title), message: "", preferredStyle: .ActionSheet)
            
            let action = UIAlertAction(title: "GeoJSON", style: .Default, handler: {
                (alert: UIAlertAction!) -> Void in
                let exporter = LayerJSONExporter(layer: layer!, localStorage: self.storage!.local)
                exporter.run()
            })
            
            let action2 = UIAlertAction(title: "KML", style: .Default, handler: {
                (alert2: UIAlertAction!) -> Void in
                alert("Not implemented yet.")
            })
            
            let cancelAction = UIAlertAction(title: "Cancel", style: .Cancel, handler: {
                (alert: UIAlertAction!) -> Void in
            })
            
            optionMenu.addAction(action)
            optionMenu.addAction(action2)
            optionMenu.addAction(cancelAction)
            
            let cell = tableView(tableView, cellForRowAtIndexPath: indexPath!)
            optionMenu.popoverPresentationController?.sourceView = cell
            optionMenu.popoverPresentationController?.sourceRect = cell.bounds
            optionMenu.popoverPresentationController?.permittedArrowDirections = UIPopoverArrowDirection.Left
            
            presentViewController(optionMenu, animated: true, completion: nil)
        }
    }
    
    func exportSelected(sender: UIBarButtonItem){
        let optionMenu = UIAlertController(title: String(format: l("Choose export format for selected layers (%@)"), String(tableView.indexPathsForSelectedRows!.count)), message: "", preferredStyle: .ActionSheet)
        
        let action = UIAlertAction(title: "GeoJSON", style: .Default, handler: {
            (alert: UIAlertAction!) -> Void in
            var paths: [NSIndexPath]?
            paths = self.tableView.indexPathsForSelectedRows as [NSIndexPath]?
            if(paths != nil){
                for path in paths!{
                    let object = self.frc!.objectAtIndexPath(path) as? Layer
                    let exporter = LayerJSONExporter(layer: object!, localStorage: self.storage!.local)
                    exporter.run()
                }
            }
            self.storage!.synchronise()
        })
        
        let action2 = UIAlertAction(title: "KML", style: .Default, handler: {
            (alert2: UIAlertAction!) -> Void in
            alert("Not implemented yet.")
        })
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .Cancel, handler: {
            (alert: UIAlertAction!) -> Void in
        })
        
        optionMenu.addAction(action)
        optionMenu.addAction(action2)
        optionMenu.addAction(cancelAction)
        
        optionMenu.popoverPresentationController?.barButtonItem = sender
        
        presentViewController(optionMenu, animated: true, completion: nil)
        
        
    }
    
    func deleteItem(layer: Layer?, indexPath: NSIndexPath?) {
        if(layer != nil && layer?.managedObjectContext != nil && indexPath != nil){
            let optionMenu = UIAlertController(title: String(format: l("Really delete layer %@?"), layer!.title), message: l("All relevant features will be deleted too."), preferredStyle: .ActionSheet)
            
            let action = UIAlertAction(title: "Delete", style: .Destructive, handler: {
                (alert: UIAlertAction!) -> Void in
                self.storage!.delete(layer!)
                self.storage!.synchronise()
                self.delegate?.didChangeData()
            })
            
            let cancelAction = UIAlertAction(title: "Cancel", style: .Default, handler: {
                (alert: UIAlertAction!) -> Void in
            })
            
            optionMenu.addAction(action)
            optionMenu.addAction(cancelAction)
            
            let cell = tableView(tableView, cellForRowAtIndexPath: indexPath!)
            optionMenu.popoverPresentationController?.sourceView = cell
            optionMenu.popoverPresentationController?.sourceRect = cell.bounds
            optionMenu.popoverPresentationController?.permittedArrowDirections = [UIPopoverArrowDirection.Up, UIPopoverArrowDirection.Down]
            
            presentViewController(optionMenu, animated: true, completion: nil)
        }
    }
    
    func showMoreActionsForSelected(sender: UIBarButtonItem) {
        let optionMenu = UIAlertController(title: nil, message: "Choose an action for selected layers:", preferredStyle: UIAlertControllerStyle.ActionSheet)
        
        let action = UIAlertAction(
            title: l("Export"),
            style: .Default,
            handler: {(alert: UIAlertAction) -> Void in
                self.exportSelected(sender)
        })
        optionMenu.addAction(action)
        
        let action2 = UIAlertAction(
            title: l("Duplicate"),
            style: .Default,
            handler: {(alert2: UIAlertAction) -> Void in
                alert("Not implemented yet.")
        })
        optionMenu.addAction(action2)
        
        let action3 = UIAlertAction(
            title: l("Download images"),
            style: .Default,
            handler: {(alert2: UIAlertAction) -> Void in
                alert("Not implemented yet.")
        })
        optionMenu.addAction(action3)
        
        let cancelAction = UIAlertAction(title: l("Cancel"), style: .Cancel, handler: {
            (alert: UIAlertAction) -> Void in
        })
        optionMenu.addAction(cancelAction)
        
        optionMenu.popoverPresentationController?.barButtonItem = sender
        
        presentViewController(optionMenu, animated: true, completion: nil)
    }
    
    // MARK: - SEGUES
    override func shouldPerformSegueWithIdentifier(identifier: String, sender: AnyObject?) -> Bool {
        if(self.editing){
            return false
        }else{
            return super.shouldPerformSegueWithIdentifier(identifier, sender: sender)
        }
    }
    
    // MARK: - TABLE VIEW
    override func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        let object = storage!.get(indexPath) as! Layer
        if(object.publicBool){
            return 105
        }else{
            return 87
        }
    }
    
    override func tableView(tableView: UITableView, canMoveRowAtIndexPath indexPath: NSIndexPath) -> Bool {
        return false
    }
    
    override func configureCell(cell: UITableViewCell, atIndexPath indexPath: NSIndexPath) {
        let object = storage!.get(indexPath) as! Layer
        
        let cell2 = cell as! LayerCell
        cell2.layerObject = object
        cell2.title!.text = object.title
        cell2.descriptionText!.text = object.descriptionString
        cell2.modification!.text = dateToJSON(object.lastEdit)
        cell2.itemsCount.text = String(object.featuresCount)
        setTypeIcon(cell2.typeIcon, type: object.kind)
        
        /// configure items for public layers
        if(object.publicBool){
            cell2.ownerLabel.hidden = false
            cell2.owner.text = object.owner
            cell2.owner.hidden = false
            cell2.sharedIcon.hidden = false
        }else{
            cell2.ownerLabel.hidden = true
            cell2.owner.hidden = true
            cell2.sharedIcon.hidden = true
        }
        
        // set tap recognizer for imageviews
        let tap3 = UITapGestureRecognizer(target: self, action: "typeIconClick:")
        cell2.typeIcon.addGestureRecognizer(tap3)
        cell2.typeIcon.userInteractionEnabled = true
        cell2.typeIcon.tag = indexPath.row
        
        let tap2 = UITapGestureRecognizer(target: self, action: "sharedIconClick:")
        cell2.sharedIcon.addGestureRecognizer(tap2)
        cell2.sharedIcon.userInteractionEnabled = true
    }
    
    override func tableView(tableView: UITableView, editActionsForRowAtIndexPath indexPath: NSIndexPath) -> [UITableViewRowAction]?  {
        let features = UITableViewRowAction(style: UITableViewRowActionStyle.Default, title: "View\nfeatures" , handler: { (action:UITableViewRowAction, indexPath:NSIndexPath) -> Void in
            // show layer features
            alert("Not implemented yet.")
        })
        features.backgroundColor = SwipeActionsColors.duplicate
        
        let edit = UITableViewRowAction(style: UITableViewRowActionStyle.Default, title: "Edit\nsettings" , handler: { (action:UITableViewRowAction, indexPath:NSIndexPath) -> Void in
            self.editItem(indexPath)
            self.setEditing(false, animated: true)
        })
        edit.backgroundColor = SwipeActionsColors.edit
        
        let more = UITableViewRowAction(style: UITableViewRowActionStyle.Default, title: "More..." , handler: { (action:UITableViewRowAction, indexPath:NSIndexPath) -> Void in
            let cell = self.tableView(tableView, cellForRowAtIndexPath: indexPath)
            self.showMoreActionsMenu(cell, indexPath: indexPath)
        })
        more.backgroundColor = SwipeActionsColors.more
        
        return [more, edit, features]
    }
    
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        //let lip = storage!.get(indexPath) as! LayerInProject
        //activeLayerInProject = lip
        //delegate?.didChangeActiveLayer(activeLayerInProject!)
        super.tableView(tableView, didSelectRowAtIndexPath: indexPath)
    }
    
    func showMoreActionsMenu(cell: UITableViewCell, indexPath: NSIndexPath) {
        let object = storage!.get(indexPath) as? Layer
        let layerName = object!.title
        let optionMenu = UIAlertController(title: nil, message: "Choose an action for the layer " + layerName, preferredStyle: UIAlertControllerStyle.ActionSheet)
        
        let action2 = UIAlertAction(
            title: l("Delete pernamently"),
            style: .Default,
            handler: {(alert: UIAlertAction) -> Void in
                self.deleteItem(object, indexPath: indexPath)
        })
        optionMenu.addAction(action2)
        
        let action3 = UIAlertAction(
            title: l("Export"),
            style: .Default,
            handler: {(alert: UIAlertAction) -> Void in
                self.exportItem(object, indexPath: indexPath)
        })
        optionMenu.addAction(action3)
        
        let action4 = UIAlertAction(
            title: l("Duplicate"),
            style: .Default,
            handler: {(alert2: UIAlertAction) -> Void in
                alert("Not implemented yet.")
        })
        optionMenu.addAction(action4)
        
        let action5 = UIAlertAction(
            title: l("Download images"),
            style: .Default,
            handler: {(alert2: UIAlertAction) -> Void in
                alert("Not implemented yet.")
        })
        optionMenu.addAction(action5)
        
        let cancelAction = UIAlertAction(title: l("Cancel"), style: .Cancel, handler: {
            (alert: UIAlertAction) -> Void in
        })
        optionMenu.addAction(cancelAction)
        
        // align the dialog to the right of the row
        tableView.setEditing(false, animated: true)
        optionMenu.popoverPresentationController?.sourceView = cell as UIView
        optionMenu.popoverPresentationController?.sourceRect = cell.bounds
        optionMenu.popoverPresentationController?.permittedArrowDirections = [UIPopoverArrowDirection.Up, UIPopoverArrowDirection.Down]
        
        // show the dialog
        presentViewController(optionMenu, animated: true, completion: nil)
    }
    
}