//
//  MapLayersController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 9/29/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import Foundation
import UIKit
import CoreData

protocol LayerInProjectControllerDelegate{
    func didChangeActiveLayer(layerInProject: LayerInProject)
    func didChangeData()
}

class LayersInProjectController: DatabaseTableViewController, LayerDeatilControllerDelegate, MapControllerDelegate {
    
    var layerStorage: LayerStorage? = nil
    var layerAttributeStorage: LayerAttributeStorage? = nil
    var featureStorage: FeatureStorage? = nil
    var detail: MapController? = nil
    var project: Project? = nil
    var delegate: LayerInProjectControllerDelegate? = nil
    var activeLayerInProject: LayerInProject? = nil
    var lastActiveLayerCell: LayerInProjectCell? = nil
    
    var enabledLayers: [Bool] = []
    
    var defaultToolbarButtons: [UIBarButtonItem] = []
    var editingToolbarButtons: [UIBarButtonItem] = []
    
    override func viewDidLoad() {
        super.viewDidLoad()
        layerAttributeStorage = LayerAttributeStorage(moc: storage!.managedObjectContext, delegate: nil)
        self.navigationItem.leftBarButtonItems = [self.editButtonItem()]
        initToolbarButtons()
        updateToolbarState()
        updateNotification!.registerForUpdates(["Project", "Layer", "LayerInProject"])
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        navigationController?.setToolbarHidden(false, animated: false)
        storage?.delegate=self
        layerStorage?.delegate=self
        //storage?.clearCache()
    }
    
    func didSaveLayer() {
        reloadData()
        delegate?.didChangeData()
    }
    
    func didCancelLayer() {
        
    }
    
    @IBAction func closeButtonClick(sender: AnyObject) {
        if(UIDevice.currentDevice().userInterfaceIdiom == .Pad){
            splitViewController?.preferredDisplayMode = UISplitViewControllerDisplayMode.PrimaryHidden
        }else{
            dismissViewControllerAnimated(true, completion: nil)
        }
    }
    
    func visibilityIconClick(recognizer: UITapGestureRecognizer){
        let image = recognizer.view as! UIImageView
        let cell = image.superview!.superview as! LayerInProjectCell
        
        cell.layerInProject!.visibility = !cell.layerInProject!.visibility
        storage!.update(cell.layerInProject!)
        
        setVisibilityIconState(image, lip: cell.layerInProject!)
        delegate?.didChangeData()
    }
    
    private func setVisibilityIconState(icon: UIImageView, lip: LayerInProject){
        if(lip.visibility){
            icon.image = UIImage(named: Images.LayerVisible)
        }else{
            icon.image = UIImage(named: Images.LayerInvisible)
        }
    }
    
    private func setTypeIcon(icon: UIImageView, type: String){
        switch type{
        case LayerTypes.point.lowercaseString:
            icon.image = UIImage(named: Images.LayerPoint)
        case LayerTypes.line.lowercaseString:
            icon.image = UIImage(named: Images.LayerLine)
        case LayerTypes.polygon.lowercaseString:
            icon.image = UIImage(named: Images.LayerPolygon)
        default:
            break
        }
    }
    
    func sharedIconClick(recognizer: UITapGestureRecognizer){
        alert(l("This layer is public."))
    }
    
    func activeLayerIconClick(recognizer: UITapGestureRecognizer){
        alert(l("This layer is active. It means new map objects will be added to this layer. You can change the active layer by tapping another one."))
    }
    
    func typeIconClick(recognizer: UITapGestureRecognizer){
        let image = recognizer.view as! UIImageView
        let view = image.superview! as UIView
        let cell = view.superview!.superview as! LayerInProjectCell
        if(cell.layerInProject != nil && cell.layerInProject!.layer != nil){
            let type = cell.layerInProject!.layer!.kind
            alert(String(format: l("This layer type is %@."), type))
        }
    }
    
    func initToolbarButtons(){
        defaultToolbarButtons = [
            UIBarButtonItem(barButtonSystemItem: UIBarButtonSystemItem.FlexibleSpace, target: nil, action: nil),
            UIBarButtonItem(title: l("+ Add layer"), style: .Plain, target: self, action: "addLayerClick:"),
            UIBarButtonItem(barButtonSystemItem: .FlexibleSpace, target: nil, action: nil)
        ]
        editingToolbarButtons = [
            UIBarButtonItem(title: l("Edit"), style: .Plain, target: self, action: "editSelectedClick"),
            UIBarButtonItem(barButtonSystemItem: UIBarButtonSystemItem.FlexibleSpace, target: nil, action: nil),
            UIBarButtonItem(title: l("Remove"), style: .Plain, target: self, action: "showRemoveOptions:"),
            UIBarButtonItem(barButtonSystemItem: UIBarButtonSystemItem.FlexibleSpace, target: nil, action: nil),
            UIBarButtonItem(title: l("More actions..."), style: .Plain, target: self, action: "showMoreActionsForSelected:")
        ]
    }
    
    override func updateToolbarState(){
        let count = tableView.indexPathsForSelectedRows?.count
        if(editing){
            self.toolbarItems = editingToolbarButtons
            (self.toolbarItems![0] as UIBarButtonItem).enabled = (count==1) // edit button only for one selected item
            (self.toolbarItems![2] as UIBarButtonItem).enabled = (count>0)
            (self.toolbarItems![4] as UIBarButtonItem).enabled = (count>0)
        }else{
            self.toolbarItems = defaultToolbarButtons
        }
    }
    
    func addLayerClick(sender: UIBarButtonItem){
        let optionMenu = UIAlertController(title: nil, message: l("Do you want to add a new layer or pin one existing?"), preferredStyle: UIAlertControllerStyle.ActionSheet)
        
        let action = UIAlertAction(
            title: l("Add new layer"),
            style: .Default,
            handler: {(alert: UIAlertAction) -> Void in
                self.addNewLayer()
        })
        optionMenu.addAction(action)
        
        let action2 = UIAlertAction(
            title: l("Add existing layer"),
            style: .Default,
            handler: {(alert: UIAlertAction) -> Void in
                self.addExistingLayer(sender)
        })
        optionMenu.addAction(action2)
        
        let cancelAction = UIAlertAction(title: l("Cancel"), style: .Cancel, handler: {
            (alert: UIAlertAction) -> Void in
        })
        optionMenu.addAction(cancelAction)
        
        optionMenu.popoverPresentationController?.barButtonItem = sender as UIBarButtonItem
        
        presentViewController(optionMenu, animated: true, completion: nil)
    }
    
    func addNewLayer(){
        if(isIPad()){
            let navCtrl = self.storyboard?.instantiateViewControllerWithIdentifier("layer-attributes-nav") as! UINavigationController
            let target = navCtrl.childViewControllers[0] as! LayerDetailWithAttributtesController
            target.storage = layerStorage
            target.layerInProjectStorage = storage as? LayerInProjectStorage
            target.layerAttributeStorage = layerAttributeStorage
            target.project = project
            target.tempLayerAttributesManager = TempLayerAttributesManager(storage: layerAttributeStorage!)
            target.delegate = self
            presentViewController(navCtrl, animated: true, completion: nil)
        }else{
            let navCtrl = self.storyboard?.instantiateViewControllerWithIdentifier("layer-detail-nav") as! UINavigationController
            let target = navCtrl.childViewControllers[0] as! LayerDetailController
            target.storage = layerStorage
            target.layerInProjectStorage = storage as? LayerInProjectStorage
            target.layerAttributeStorage = layerAttributeStorage
            target.project = project
            target.tempLayerAttributesManager = TempLayerAttributesManager(storage: layerAttributeStorage!)
            target.delegate = self
            presentViewController(navCtrl, animated: true, completion: nil)
        }
    }
    
    func addExistingLayer(sender: UIBarButtonItem){
        let navCtrl = self.storyboard?.instantiateViewControllerWithIdentifier("pin-layer-nav") as! UINavigationController
        let target = navCtrl.childViewControllers[0] as! PinLayerController
        target.managedObjectContext = storage!.managedObjectContext
        target.layerInProjectStorage = storage as? LayerInProjectStorage
        target.project = project
        target.lastOrder = getAll()!.fetchedObjects!.count
        target.delegate = self
        
        navCtrl.modalPresentationStyle = .Popover
        navCtrl.popoverPresentationController?.barButtonItem = sender
        navCtrl.popoverPresentationController?.permittedArrowDirections = UIPopoverArrowDirection.Left
        
        presentViewController(navCtrl, animated: true, completion: nil)
    }
    
    func editSelectedClick() {
        if let indexPath = self.tableView.indexPathForSelectedRow {
            editItem(indexPath)
        }
    }
    
    func editItem(indexPath: NSIndexPath){
        if(isIPad()){
            let navCtrl = self.storyboard?.instantiateViewControllerWithIdentifier("layer-attributes-nav") as! UINavigationController
            let target = navCtrl.childViewControllers[0] as! LayerDetailWithAttributtesController
            target.storage = layerStorage
            target.layerInProjectStorage = storage as? LayerInProjectStorage
            target.layerAttributeStorage = layerAttributeStorage
            let object = frc!.objectAtIndexPath(indexPath) as! LayerInProject
            target.layer = object.layer
            target.project = project
            target.layerInProject = object
            let tempLayerAttributesManager = TempLayerAttributesManager(storage: layerAttributeStorage!)
            tempLayerAttributesManager.load(object.layer!)
            target.tempLayerAttributesManager = tempLayerAttributesManager
            target.delegate = self
            presentViewController(navCtrl, animated: true, completion: nil)
        }else{
            let navCtrl = self.storyboard?.instantiateViewControllerWithIdentifier("layer-detail-nav") as! UINavigationController
            let target = navCtrl.childViewControllers[0] as! LayerDetailController
            target.storage = layerStorage
            target.layerInProjectStorage = storage as? LayerInProjectStorage
            target.layerAttributeStorage = layerAttributeStorage
            let object = frc!.objectAtIndexPath(indexPath) as! LayerInProject
            target.layer = object.layer
            target.project = project
            target.layerInProject = object
            let tempLayerAttributesManager = TempLayerAttributesManager(storage: layerAttributeStorage!)
            tempLayerAttributesManager.load(object.layer!)
            target.tempLayerAttributesManager = tempLayerAttributesManager
            target.delegate = self
            presentViewController(navCtrl, animated: true, completion: nil)
        }
    }
    
    func unpinItem(object: LayerInProject?){
        if(object != nil && object?.managedObjectContext != nil){
            self.storage!.delete(object!)
        }
    }
    
    func unpinSelected(){
        var paths: [NSIndexPath]?
        repeat {
            paths = tableView.indexPathsForSelectedRows as [NSIndexPath]?
            if(paths != nil){
                if(paths != nil){
                    let object = frc!.objectAtIndexPath(paths![0]) as? LayerInProject
                    unpinItem(object)
                }
            }
        } while paths != nil
        self.storage!.synchronise()
    }
    
    func showRemoveOptions(sender: UIBarButtonItem) {
        let optionMenu = UIAlertController(title: nil, message: l("Do you want to unpin the layer(s) from the project or delete it pernamently?"), preferredStyle: UIAlertControllerStyle.ActionSheet)
        
        let action = UIAlertAction(
            title: l("Unpin from project"),
            style: .Default,
            handler: {(alert: UIAlertAction) -> Void in
                self.unpinSelected()
                self.delegate?.didChangeData()
        })
        optionMenu.addAction(action)
        
        let action2 = UIAlertAction(
            title: l("Delete pernamently"),
            style: .Default,
            handler: {(alert: UIAlertAction) -> Void in
                self.deleteSelected(sender)
                self.delegate?.didChangeData()
        })
        optionMenu.addAction(action2)
        
        let cancelAction = UIAlertAction(title: l("Cancel"), style: .Cancel, handler: {
            (alert: UIAlertAction) -> Void in
        })
        optionMenu.addAction(cancelAction)
        
        optionMenu.popoverPresentationController?.barButtonItem = sender
        
        presentViewController(optionMenu, animated: true, completion: nil)
    }
    
    override func deleteSelected(sender: UIBarButtonItem) {
        let paths = tableView.indexPathsForSelectedRows
        let items = databaseActions!.getItemsForPaths(paths)
        
        if(items.count > 0){
            let optionMenu = UIAlertController(title: String(format: l("Really delete %u items?"), paths!.count), message: l("All included content will be deleted too."), preferredStyle: .ActionSheet)
            
            let action = UIAlertAction(title: "Delete", style: .Destructive, handler: {
                (alert: UIAlertAction!) -> Void in
                self.deleteItems(items)
            })
            
            let cancelAction = UIAlertAction(title: "Cancel", style: .Default, handler: {
                (alert: UIAlertAction!) -> Void in
            })
            
            optionMenu.addAction(action)
            optionMenu.addAction(cancelAction)
            optionMenu.popoverPresentationController?.barButtonItem = sender
            
            presentViewController(optionMenu, animated: true, completion: nil)
        }
    }
    
    private func deleteItems(items: [NSManagedObject]){
        for object in items{
            if(object.managedObjectContext != nil){
                let lip = object as! LayerInProject
                layerStorage!.delete(lip.layer!)
            }
        }
        self.layerStorage!.synchronise()
    }
    
    func deleteItem(lip: LayerInProject?, indexPath: NSIndexPath?) {
        if(lip != nil && lip?.managedObjectContext != nil && lip?.layer?.managedObjectContext != nil && indexPath != nil){
            // show confirm dialog
            let layer: Layer? = lip!.layer
            if(layer != nil){
                let optionMenu = UIAlertController(title: String(format: l("Really delete layer %@?"), layer!.title), message: l("All relevant features will be deleted too."), preferredStyle: .ActionSheet)
                
                let action = UIAlertAction(title: "Delete", style: .Destructive, handler: {
                    (alert: UIAlertAction!) -> Void in
                    self.layerStorage!.delete(layer!)
                    self.layerStorage!.synchronise()
                    self.delegate?.didChangeData()
                })
                
                let cancelAction = UIAlertAction(title: "Cancel", style: .Default, handler: {
                    (alert: UIAlertAction!) -> Void in
                })
                
                optionMenu.addAction(action)
                optionMenu.addAction(cancelAction)
                
                let cell = tableView(tableView, cellForRowAtIndexPath: indexPath!)
                optionMenu.popoverPresentationController?.sourceView = cell
                optionMenu.popoverPresentationController?.sourceRect = cell.bounds
                optionMenu.popoverPresentationController?.permittedArrowDirections = UIPopoverArrowDirection.Left
                
                presentViewController(optionMenu, animated: true, completion: nil)
            }
        }
    }
    
    func exportItem(lip: LayerInProject?, indexPath: NSIndexPath?) {
        if(lip != nil && lip?.managedObjectContext != nil && lip?.layer?.managedObjectContext != nil && indexPath != nil){
            // show confirm dialog
            let layer: Layer? = lip!.layer
            if(layer != nil){
                let optionMenu = UIAlertController(title: String(format: l("Choose export format for layer %@?"), layer!.title), message: "", preferredStyle: .ActionSheet)
                
                let action = UIAlertAction(title: "GeoJSON", style: .Default, handler: {
                    (alert: UIAlertAction!) -> Void in
                    let exporter = LayerJSONExporter(layer: layer!, localStorage: self.storage!.local)
                    exporter.run()
                })
                
                let action2 = UIAlertAction(title: "KML", style: .Default, handler: {
                    (alert2: UIAlertAction!) -> Void in
                    alert("Not implemented yet.")
                })
                
                let cancelAction = UIAlertAction(title: "Cancel", style: .Cancel, handler: {
                    (alert: UIAlertAction!) -> Void in
                })
                
                optionMenu.addAction(action)
                optionMenu.addAction(action2)
                optionMenu.addAction(cancelAction)
                
                let cell = tableView(tableView, cellForRowAtIndexPath: indexPath!)
                optionMenu.popoverPresentationController?.sourceView = cell
                optionMenu.popoverPresentationController?.sourceRect = cell.bounds
                optionMenu.popoverPresentationController?.permittedArrowDirections = UIPopoverArrowDirection.Left
                
                presentViewController(optionMenu, animated: true, completion: nil)
            }
        }
    }
    
    func exportSelected(sender: UIBarButtonItem){
        let optionMenu = UIAlertController(title: String(format: l("Choose export format for selected layers (%@)"), String(tableView.indexPathsForSelectedRows!.count)), message: "", preferredStyle: .ActionSheet)
        
        let action = UIAlertAction(title: "GeoJSON", style: .Default, handler: {
            (alert: UIAlertAction!) -> Void in
            var paths: [NSIndexPath]?
            paths = self.tableView.indexPathsForSelectedRows as [NSIndexPath]?
            if(paths != nil){
                for path in paths!{
                    let object = self.frc!.objectAtIndexPath(path) as? LayerInProject
                    let exporter = LayerJSONExporter(layer: object!.layer!, localStorage: self.storage!.local)
                    exporter.run()
                }
            }
            self.storage!.synchronise()
        })
        
        let action2 = UIAlertAction(title: "KML", style: .Default, handler: {
            (alert2: UIAlertAction!) -> Void in
            alert("Not implemented yet.")
        })
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .Cancel, handler: {
            (alert: UIAlertAction!) -> Void in
        })
        
        optionMenu.addAction(action)
        optionMenu.addAction(action2)
        optionMenu.addAction(cancelAction)
        
        optionMenu.popoverPresentationController?.barButtonItem = sender
        
        presentViewController(optionMenu, animated: true, completion: nil)
        
        
    }
    
    func showMoreActionsForSelected(sender: UIBarButtonItem) {
        let optionMenu = UIAlertController(title: nil, message: "Choose an action for selected layers:", preferredStyle: UIAlertControllerStyle.ActionSheet)

        let action = UIAlertAction(
            title: l("Export"),
            style: .Default,
            handler: {(alert: UIAlertAction) -> Void in
                self.exportSelected(sender)
        })
        optionMenu.addAction(action)
        
        let action2 = UIAlertAction(
            title: l("Duplicate"),
            style: .Default,
            handler: {(alert2: UIAlertAction) -> Void in
                alert("Not implemented yet.")
        })
        optionMenu.addAction(action2)
        
        let action3 = UIAlertAction(
            title: l("Download images"),
            style: .Default,
            handler: {(alert2: UIAlertAction) -> Void in
                alert("Not implemented yet.")
        })
        optionMenu.addAction(action3)
        
        let cancelAction = UIAlertAction(title: l("Cancel"), style: .Cancel, handler: {
            (alert: UIAlertAction) -> Void in
        })
        optionMenu.addAction(cancelAction)
        
        optionMenu.popoverPresentationController?.barButtonItem = sender
        
        presentViewController(optionMenu, animated: true, completion: nil)
    }
    
    // MARK: - SEGUES
    override func shouldPerformSegueWithIdentifier(identifier: String, sender: AnyObject?) -> Bool {
        if(self.editing){
            return false
        }else{
            return super.shouldPerformSegueWithIdentifier(identifier, sender: sender)
        }
    }
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if(segue.identifier=="close"){
            let dst = segue.destinationViewController as! MapController
            dst.managedObjectContext = managedObjectContext
        }
    }
    
    // MARK: - TABLE VIEW
    override func getAll() -> NSFetchedResultsController? {
        if(frc == nil){
            frc = (storage as! LayerInProjectStorage).getAllForProject(project!)
        }
        return frc
    }
    
    override func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        let object = frc?.objectAtIndexPath(indexPath) as! LayerInProject
        if(object.layer != nil && object.layer!.publicBool){
            return 105
        }else{
            return 87
        }
    }
    
    override func tableView(tableView: UITableView, canMoveRowAtIndexPath indexPath: NSIndexPath) -> Bool {
        return true
    }
    
    override func tableView(tableView: UITableView, moveRowAtIndexPath fromIndexPath: NSIndexPath, toIndexPath: NSIndexPath) {
        
        let from = Int(fromIndexPath.row)
        let to = Int(toIndexPath.row)
        
        // if changed
        if(from != to){
            
            // direction of movement
            let s = (from < to) ? 1 : -1
            
            // fetch all layers first
            let all = getAll()!.fetchedObjects
            
            // update order of all layers, it fixes spaces after deleted items too
            for(var row = 0; row < tableView.numberOfRowsInSection(0); row++){
                
                if let lip = all![row] as? LayerInProject{
                    // update order
                    if((row * s) > (from * s) && (row * s) <= (to * s)){ // intermediate items between from and to rows
                        lip.order = row - s
                    }else if(row == from){ // moved item
                        lip.order = to
                    }else{ // other items
                        lip.order = row
                    }
                    // save
                    storage?.update(lip)
                }
                
            }
            
            // sync
            storage?.synchronise()
            delegate?.didChangeData()
            reloadData()
        }
    }
    
    override func configureCell(cell: UITableViewCell, atIndexPath indexPath: NSIndexPath) {
        let cell2 = cell as! LayerInProjectCell
        let lip = frc?.objectAtIndexPath(indexPath) as? LayerInProject
        
        if(lip != nil){
        
            let objecto = lip!.layer
            if(objecto == nil){
                return Void()
            }
            
            let object = objecto! as Layer
            
            cell2.layerInProject = lip
            cell2.title!.text = object.title
            cell2.descriptionText!.text = object.descriptionString
            cell2.modification!.text = dateToJSON(object.lastEdit)
            cell2.itemsCount.text = String(object.featuresCount)
            cell2.colorStrip.backgroundColor = UIColor(hexString: lip!.color!)
            setTypeIcon(cell2.typeIcon, type: object.kind)
            setVisibilityIconState(cell2.visibilityIcon, lip: lip!)
            
            // configure items for public layers
            if(object.publicBool){
                cell2.ownerLabel.hidden = false
                cell2.owner.text = object.owner
                cell2.owner.hidden = false
                cell2.sharedIcon.hidden = false
            }else{
                cell2.ownerLabel.hidden = true
                cell2.owner.hidden = true
                cell2.sharedIcon.hidden = true
            }
            
            // set tap recognizer for imageviews
            let tap3 = UITapGestureRecognizer(target: self, action: "typeIconClick:")
            cell2.typeIcon.addGestureRecognizer(tap3)
            cell2.typeIcon.userInteractionEnabled = true
            cell2.typeIcon.tag = indexPath.row
            
            let tap = UITapGestureRecognizer(target: self, action: "visibilityIconClick:")
            cell2.visibilityIcon.addGestureRecognizer(tap)
            cell2.visibilityIcon.userInteractionEnabled = true
            cell2.visibilityIcon.tag = indexPath.row
            
            let tap2 = UITapGestureRecognizer(target: self, action: "sharedIconClick:")
            cell2.sharedIcon.addGestureRecognizer(tap2)
            cell2.sharedIcon.userInteractionEnabled = true
            
            let tap4 = UITapGestureRecognizer(target: self, action: "activeLayerIconClick:")
            cell2.activeLayerIcon.addGestureRecognizer(tap4)
            cell2.activeLayerIcon.userInteractionEnabled = true
            
            // Select the active layer as selected. If activeLayer is nil then select the first one.
            if(activeLayerInProject == nil && indexPath.row == 0){
                cell2.activeLayerIcon.hidden = false
                //tableView.selectRowAtIndexPath(indexPath, animated: false, scrollPosition: UITableViewScrollPosition.Middle)
                activeLayerInProject = lip
                lastActiveLayerCell = cell2
                delegate?.didChangeActiveLayer(activeLayerInProject!)
            }else if(lip == activeLayerInProject){
                cell2.activeLayerIcon.hidden = false
                lastActiveLayerCell = cell2
                //tableView.selectRowAtIndexPath(indexPath, animated: false, scrollPosition: UITableViewScrollPosition.Middle)
            }else{
                cell2.activeLayerIcon.hidden = true
            }
            
        }else{
            cell2.title!.text = "Unknown"
        }
    }
    
    override func tableView(tableView: UITableView, editActionsForRowAtIndexPath indexPath: NSIndexPath) -> [UITableViewRowAction]?  {
        let features = UITableViewRowAction(style: UITableViewRowActionStyle.Default, title: "View\nfeatures" , handler: { (action:UITableViewRowAction, indexPath:NSIndexPath) -> Void in
            // show layer features
            alert("Not implemented yet.")
        })
        features.backgroundColor = SwipeActionsColors.duplicate
        
        let edit = UITableViewRowAction(style: UITableViewRowActionStyle.Default, title: "Edit\nsettings" , handler: { (action:UITableViewRowAction, indexPath:NSIndexPath) -> Void in
            self.editItem(indexPath)
            self.setEditing(false, animated: true)
        })
        edit.backgroundColor = SwipeActionsColors.edit
        
        let more = UITableViewRowAction(style: UITableViewRowActionStyle.Default, title: "More..." , handler: { (action:UITableViewRowAction, indexPath:NSIndexPath) -> Void in
            let cell = self.tableView(tableView, cellForRowAtIndexPath: indexPath)
            self.showMoreActionsMenu(cell, indexPath: indexPath)
        })
        more.backgroundColor = SwipeActionsColors.more
        
        return [more, edit, features]
    }
    
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        if(self.editing == false){ // do not change active layer in editing mode
            // hide active icon for the last active layer
            lastActiveLayerCell?.activeLayerIcon.hidden = true
            
            // change active layer
            let lip = frc?.objectAtIndexPath(indexPath) as! LayerInProject
            activeLayerInProject = lip
            delegate?.didChangeActiveLayer(activeLayerInProject!)
            
            // display active icon for the new selected active layer
            let cell = tableView.cellForRowAtIndexPath(indexPath) as! LayerInProjectCell
            cell.activeLayerIcon.hidden = false
            lastActiveLayerCell = cell
        }
        
        super.tableView(tableView, didSelectRowAtIndexPath: indexPath)
    }
    
    func showMoreActionsMenu(cell: UITableViewCell, indexPath: NSIndexPath) {
        let layerName = (frc!.objectAtIndexPath(indexPath) as! LayerInProject).layer!.title
        let optionMenu = UIAlertController(title: nil, message: "Choose an action for the layer " + layerName, preferredStyle: UIAlertControllerStyle.ActionSheet)
        let object = frc!.objectAtIndexPath(indexPath) as? LayerInProject
        
        let action = UIAlertAction(
            title: l("Unpin from project"),
            style: .Default,
            handler: {(alert: UIAlertAction) -> Void in
                self.unpinItem(object)
        })
        optionMenu.addAction(action)
        
        let action2 = UIAlertAction(
            title: l("Delete pernamently"),
            style: .Default,
            handler: {(alert: UIAlertAction) -> Void in
                self.deleteItem(object, indexPath: indexPath)
        })
        optionMenu.addAction(action2)
        
        let action3 = UIAlertAction(
            title: l("Export"),
            style: .Default,
            handler: {(alert: UIAlertAction) -> Void in
                self.exportItem(object, indexPath: indexPath)
        })
        optionMenu.addAction(action3)
        
        let action4 = UIAlertAction(
            title: l("Duplicate"),
            style: .Default,
            handler: {(alert2: UIAlertAction) -> Void in
                alert("Not implemented yet.")
        })
        optionMenu.addAction(action4)
        
        let action5 = UIAlertAction(
            title: l("Download images"),
            style: .Default,
            handler: {(alert2: UIAlertAction) -> Void in
                alert("Not implemented yet.")
        })
        optionMenu.addAction(action5)
        
        let cancelAction = UIAlertAction(title: l("Cancel"), style: .Cancel, handler: {
            (alert: UIAlertAction) -> Void in
        })
        optionMenu.addAction(cancelAction)
        
        // align the dialog to the right of the row
        tableView.setEditing(false, animated: true)
        optionMenu.popoverPresentationController?.sourceView = cell as UIView
        optionMenu.popoverPresentationController?.sourceRect = cell.bounds
        optionMenu.popoverPresentationController?.permittedArrowDirections = UIPopoverArrowDirection.Left
        
        // show the dialog
        presentViewController(optionMenu, animated: true, completion: nil)
    }

}