//
//  LocalStorage.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 9/15/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import Foundation
import CoreData

class LocalStorage{
    
    let managedObjectContext: NSManagedObjectContext
    
    // cache
    var _fetchedResultsController: NSFetchedResultsController? = nil
    var _lastEntity: String = ""
    var _lastPredicate: NSPredicate? = nil
    var _lastSort = [NSSortDescriptor]()
    
    init(managedObjectContext: NSManagedObjectContext){
        self.managedObjectContext=managedObjectContext
    }
    
    func get(indexPath: NSIndexPath)->NSManagedObject?{
        return _fetchedResultsController?.objectAtIndexPath(indexPath) as? NSManagedObject
    }
    
    func getAll(entity: String, predicate: NSPredicate?, sort: [NSSortDescriptor], delegate: NSFetchedResultsControllerDelegate?)->NSFetchedResultsController{
        
        // Returns cached results if cache exists and if parameters are same as in the cache.
        if _fetchedResultsController != nil && _lastEntity == entity && _lastPredicate == predicate && _lastSort == sort{
            return _fetchedResultsController!
        }
        
        let fetchRequest = NSFetchRequest()
        // Edit the entity name as appropriate.
        let entity = NSEntityDescription.entityForName(entity, inManagedObjectContext: self.managedObjectContext)
        fetchRequest.entity = entity

        // Set the batch size to a suitable number.
        fetchRequest.fetchBatchSize = 20
        
        // Predicate
        fetchRequest.predicate = predicate
        
        // Edit the sort key as appropriate.
        fetchRequest.sortDescriptors = sort
        
        fetchRequest.returnsObjectsAsFaults = false
        
        // Edit the section name key path and cache name if appropriate.
        // nil for section name key path means "no sections".
        let aFetchedResultsController = NSFetchedResultsController(fetchRequest: fetchRequest, managedObjectContext: self.managedObjectContext, sectionNameKeyPath: nil, cacheName: nil) // cache "Master"
        aFetchedResultsController.delegate = delegate
        _fetchedResultsController = aFetchedResultsController
        
        var error: NSError? = nil
        do {
            try _fetchedResultsController!.performFetch()
        } catch let error1 as NSError {
            error = error1
            // Replace this implementation with code to handle the error appropriately.
            // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
            print("Unresolved error \(error), \(error!.userInfo)")
            abort()
        }
        
        return _fetchedResultsController!
    }
    
    func clearCache(){
        _fetchedResultsController = nil
    }
    
    func newEntity(name: String) -> NSManagedObject{
        return NSEntityDescription.insertNewObjectForEntityForName(name, inManagedObjectContext: managedObjectContext) as NSManagedObject
    }
    
    func create(object: NSManagedObject){
        saveChanges()
    }
    
    func hide(object: NSManagedObject){
        if(object.managedObjectContext != nil){
            object.setValue(true, forKey: "hidden")
            saveChanges()
        }
    }
    
    func unhide(object: NSManagedObject){
        if(object.managedObjectContext != nil){
            object.setValue(false, forKey: "hidden")
            saveChanges()
        }
    }
    
    func update(object: NSManagedObject){
        saveChanges()
    }
    
    func delete(object: NSManagedObject){
        managedObjectContext.deleteObject(object)
        saveChanges()
    }
    
    func saveChanges(){
        var error: NSError? = nil
        do {
            try managedObjectContext.save()
        } catch let error1 as NSError {
            error = error1
            print("Unresolved error \(error), \(error?.description)", terminator: "")
            abort()
        }
    }

}
