//
//  MapScaleBarController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 15/02/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import Foundation
import UIKit
import GoogleMaps

/// Calculating and updating map scale measure according to the current map camera position. It uses measure levels starting with 1, 2 or 5 and two units - meters and kilometers.
class MapScaleBarView: UIView {
    
    @IBOutlet weak var scaleText: UILabel!
    @IBOutlet weak var zoomText: UILabel!
    @IBOutlet weak var width: NSLayoutConstraint!
    
    /// Minimal width of the scale bar in pixels. Scale bar is enlarged up to 2.5 times.
    let minWidth: Double = 50
    
    /**
     Updates UI elements of the scale bar according to the current map state.
     */
    func update(zoom: Float, mapProjection: GMSProjection){
        // convert minWidth form pixels to real distance in meters, using a horizontal line representing the scale bar view
        let left = CGPoint(x: self.frame.minX, y: self.frame.midY)
        let right = CGPoint(x: self.frame.minX.advancedBy(CGFloat(minWidth)), y: self.frame.midY)
        let from = mapProjection.coordinateForPoint(left)
        let to = mapProjection.coordinateForPoint(right)
        let dst = Double(GMSGeometryDistance(from, to))
        
        // round distance to the nearest upper number starting with 1, 2 or 5
        let n = String(Int(round(dst))).characters.count - 1 // digits of distance integer value subtracted by 1
        let zeros = pow(10.0, Double(n))
        let f = dst / zeros // distance with dot after the first digit
        var result: Double = 0
        
        if(f > 1.0 && f <= 2.0){
            result = 2 * zeros
        }else if(f > 2.0 && f <= 5.0){
            result = 5 * zeros
        }else{
            result = 10 * zeros
        }
        
        // update scale bar width like a multiplier of minWidth
        let newWidthValue = CGFloat(minWidth * (result / dst))
        self.removeConstraint(width)
        width = NSLayoutConstraint(item: self, attribute: NSLayoutAttribute.Width, relatedBy: NSLayoutRelation.Equal, toItem: nil, attribute: NSLayoutAttribute.Width, multiplier: 1, constant: newWidthValue)
        self.addConstraint(width)
        
        // convert units
        var unit = "m"
        if(result > 1000){
            unit = "km"
            result /= 1000
        }
        
        // update texts
        scaleText.text = String(format: "%.1f %@", result, unit)
        zoomText.text = String(format: l("Zoom: %.1f"), zoom)
    }
}

