//
//  ViewController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 9/6/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import UIKit
import CoreData

class ProjectDetailController: BaseTableViewController, DefaultFormValidatorDelegate {
    
    var project: Project? = nil
    var storage: ProjectStorage? = nil
    var validator: DefaultFormValidator? = nil
    
    @IBOutlet weak var nameField: UITextField!
    @IBOutlet weak var descriptionField: UITextField!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        
        if(project == nil){
            baseActions!.setTitleText(NSLocalizedString("Add map", comment: ""))
        }else{
            baseActions!.setTitleText(l("Edit map"))
            loadValues(project!)
        }
        
        initValidation()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    private func initValidation(){
        validator = DefaultFormValidator()
        validator!.requiredField(nameField)
    }
    
    @IBAction func doneClick(sender: AnyObject) {
        validator?.validate(self)
    }
    
    @IBAction func cancelClick(sender: AnyObject) {
        self.dismissViewControllerAnimated(true, completion: { () -> Void in
            
        })
    }
    
    func validationSuccessful() {
        saveMap()
        self.dismissViewControllerAnimated(true, completion: { () -> Void in
            
        })
    }
    
    private func loadValues(map: Project){
        nameField.text=map.title
        descriptionField.text=map.descriptionString
    }
    
    private func saveMap(){
        if(checkForm()){
            if(storage != nil){
                // unique check
                if(project?.title != nameField.text && storage!.getByTitle(nameField.text!) != nil){
                    alert(l("The entered name must be unique!"))
                }else{
                    if(project == nil){
                        // add new
                        project = storage!.newEntity() as? Project
                        project!.title = nameField.text!.trim
                        project!.descriptionString = descriptionField.text!.trim
                        storage?.create(project!)
                    }else{
                        // edit item
                        if(project?.managedObjectContext == nil){
                            alert(l("Map cannot be saved. Item has already been deleted!"))
                            self.dismissViewControllerAnimated(true, completion: nil)
                        }else{
                            // edit existing
                            project!.title = nameField.text!.trim
                            project!.descriptionString = descriptionField.text!.trim
                            storage?.update(project!)
                        }
                    }
                    storage?.synchronise()
                }
            }
        }else{
            alert("Please fill in all required fields!")
        }
    }
    
    private func checkForm()->Bool{
        nameField.text = nameField.text!.trim
        if(nameField.text! == ""){
            return false
        }
        return true
    }
}

