//
//  ProjectStorage.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 9/15/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import Foundation
import CoreData
import UIKit
import SwiftyJSON

class ProjectStorage: CentralStorage{
    
    let ENTITY_NAME = Project.getEntityName()
    
    required init(moc: NSManagedObjectContext, delegate: NSFetchedResultsControllerDelegate? = nil) {
        super.init(moc: moc, delegate: delegate, entityName: ENTITY_NAME)
    }
    
    override func getCreateURL(item: NSManagedObject) -> String {
        return "/project"
    }
    
    override func getItemURL(item: NSManagedObject) -> String {
        let project = item as! Project
        return "/project/"+project.scheme+"/"+project.name
    }
    
    override func getItemByURLParts(urlParts: [String]) -> NSManagedObject? {
        if((urlParts.count)==4){
            let name = urlParts[3]
            return getByName(name)
        }
        return nil
    }
    
    // MARK: - GENERAL
    override func getMapping() -> NSDictionary? {
        return [
            "name": "name",
            "scheme": "scheme",
            "hash": "hashString",
            "properties": [
                "title": "title",
                "description": "descriptionString",
                "public": "publicBool",
                "last_latitude": "lastLatitude",
                "last_longitude": "lastLongitude",
                "last_zoom_level": "lastZoomLevel"
            ]
        ]
    }
    
    override func getDefaultSort() -> [NSSortDescriptor] {
        return [NSSortDescriptor(key: "title", ascending: true)]
    }

}
