//
//  ViewController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 9/6/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import UIKit
import CoreData
import SwiftHTTP

protocol ProjectOldChangeDelegate{
    func didOpenProject(project: Project)
    func didCloseProject()
}

class ProjectsControllerOld: DatabaseTableViewController, WelcomePageControllerDelegate{
    
    @IBOutlet weak var editButton: UIBarButtonItem!
    @IBOutlet weak var duplicateButton: UIBarButtonItem!
    @IBOutlet weak var deleteButton: UIBarButtonItem!
    
    var layerStorage: LayerStorage? = nil
    var layerInProjectStorage: LayerInProjectStorage? = nil
    var projectChangeDelegate: ProjectChangeDelegate? = nil
    var featureStorage: FeatureStorage? = nil

    override func viewDidLoad() {
        super.viewDidLoad()
        
        //showWelcomeScreen(self)
        //configureView()
        
        storage = ProjectStorage(moc: managedObjectContext!, delegate: self)
        layerStorage = LayerStorage(moc: managedObjectContext!, delegate: nil)
        layerInProjectStorage = LayerInProjectStorage(moc: managedObjectContext!, delegate: nil)
        featureStorage = FeatureStorage(moc: managedObjectContext!)
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        navigationController?.setToolbarHidden(true, animated: false)
        reloadData()
        
        // set openned project to nil - all projects are closed if we are here
        projectChangeDelegate?.didCloseProject()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    // MARK: - Local functions
    /*
    private func configureView(){
        self.navigationItem.leftBarButtonItem = self.editButtonItem()
        
        if(settings.boolForKey(SettingsConstants.DEBUG_MODE)){
            showDebugTools()
        }
    }
    
    override func syncBeginSuccessful(){
        super.syncBeginSuccessful()
        storage?.clearCache()
        layerStorage?.clearCache()
        layerInProjectStorage?.clearCache()
        reloadData()
    }
    */
    override func updateToolbarState(){
        let count = tableView.indexPathsForSelectedRows?.count
        if(count>0 && editing){
            navigationController?.setToolbarHidden(false, animated: true)
            editButton.enabled=(count==1)
            duplicateButton.enabled=true
            deleteButton.enabled=true
        }else{
            navigationController?.setToolbarHidden(true, animated: true)
            editButton.enabled=false
            duplicateButton.enabled=false
            deleteButton.enabled=false
        }
    }
    
    @IBAction func editClick(sender: AnyObject) {
        if let indexPath = self.tableView.indexPathForSelectedRow {
            editItem(indexPath)
        }
    }
    
    func editItem(indexPath: NSIndexPath?){
        let navCtrl = self.storyboard?.instantiateViewControllerWithIdentifier("map-detail") as! UINavigationController
        let target = navCtrl.childViewControllers[0] as! ProjectDetailController
        target.storage = storage as? ProjectStorage
        let object = storage!.get(indexPath!) as! Project
        target.project = object
        presentViewController(navCtrl, animated: true, completion: nil)
    }
    
    @IBAction func duplicateClick(sender: AnyObject) {
        alert("Not implemented yet!")
    }
    
    @IBAction func deleteClick(sender: AnyObject) {
        deleteSelected(sender as! UIBarButtonItem)
    }
    
    func deleteItem(indexPath: NSIndexPath?){
        if(indexPath != nil){
            let object = storage!.get(indexPath!) as! Project?
            if(object != nil){
                self.tableView.editing = false
                
                let optionMenu = UIAlertController(title: String(format: "Really delete map %@?", object!.title), message: l("All relevant content will be deleted too."), preferredStyle: .ActionSheet)
                
                let action = UIAlertAction(title: "Delete", style: .Destructive, handler: {
                    (alert: UIAlertAction!) -> Void in
                    self.storage!.delete(object!)
                    self.storage!.synchronise()
                })
                
                let cancelAction = UIAlertAction(title: "Cancel", style: .Default, handler: {
                    (alert: UIAlertAction!) -> Void in
                })
                
                optionMenu.addAction(action)
                optionMenu.addAction(cancelAction)
                
                let cell = tableView(tableView, cellForRowAtIndexPath: indexPath!)
                optionMenu.popoverPresentationController?.sourceView = cell
                optionMenu.popoverPresentationController?.sourceRect = cell.bounds
                optionMenu.popoverPresentationController?.permittedArrowDirections = UIPopoverArrowDirection.Up
                
                presentViewController(optionMenu, animated: true, completion: nil)
            }
        }
    }
    
    // MARK: - Segues
    override func shouldPerformSegueWithIdentifier(identifier: String, sender: AnyObject?) -> Bool {
        if(self.editing){
            return false
        }else{
            return super.shouldPerformSegueWithIdentifier(identifier, sender: sender)
        }
    }
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if segue.identifier == "add-map" {
            let navi = segue.destinationViewController as! UINavigationController
            let target = navi.viewControllers[0] as! ProjectDetailController
            target.storage = storage as? ProjectStorage
        }
    }
 
    // MARK: - TABLEVIEW
    
    override func configureCell(cell: UITableViewCell, atIndexPath indexPath: NSIndexPath) {
        let mapCell = cell 
        let object = storage!.get(indexPath) as! Project
        mapCell.textLabel!.text = object.title
        mapCell.detailTextLabel!.text = object.descriptionString
    }
    
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        super.tableView(tableView, didSelectRowAtIndexPath: indexPath)
        
        // open selected project, if not editing mode enabled
        if(!editing){
            
            // clear cache
            layerInProjectStorage?.clearCache()
            
            // set the selected project as currently openned one
            if let indexPath = self.tableView.indexPathForSelectedRow {
                let project = storage?.get(indexPath) as! Project
                projectChangeDelegate?.didOpenProject(project)
            }
            
            // different layouts - split for ipad, classic + popover for iphone
            if(isIPad()){
                let ctrl = self.storyboard?.instantiateViewControllerWithIdentifier("map-split-view-controller") as! UISplitViewController
                
                // map
                let navi2 = ctrl.viewControllers[1] as! UINavigationController
                let target = navi2.viewControllers[0] as! MapController
                if let indexPath = self.tableView.indexPathForSelectedRow {
                    let object = storage?.get(indexPath) as! Project
                    target.projectData = object
                }
                target.managedObjectContext = managedObjectContext
                target.projectStorage = storage as? ProjectStorage
                target.layerStorage = layerStorage
                target.layerInProjectStorage = layerInProjectStorage
                target.featureStorage = featureStorage
                
                // layers in project
                let target2Navi = ctrl.viewControllers[0] as! UINavigationController
                let target2 = target2Navi.viewControllers[0] as! LayersInProjectController
                if let indexPath = self.tableView.indexPathForSelectedRow {
                    let object = storage?.get(indexPath) as! Project
                    target2.project = object
                }
                target2.managedObjectContext = managedObjectContext
                target2.layerStorage = layerStorage
                target2.storage = layerInProjectStorage
                target2.featureStorage = featureStorage
                target2.delegate = target
                
                target.delegate = target2
                
                presentViewController(ctrl, animated: true, completion: nil)
            }else{
                //map
                let ctrl = self.storyboard?.instantiateViewControllerWithIdentifier("map-view-controller") as! UINavigationController
                let target = ctrl.viewControllers[0] as! MapController
                if let indexPath = self.tableView.indexPathForSelectedRow {
                    let object = storage?.get(indexPath) as! Project
                    target.projectData = object
                }
                target.managedObjectContext = managedObjectContext
                target.projectStorage = storage as? ProjectStorage
                target.layerStorage = layerStorage
                target.layerInProjectStorage = layerInProjectStorage
                target.featureStorage = featureStorage
                
                presentViewController(ctrl, animated: true, completion: nil)
            }
        }
    }
    
    override func tableView(tableView: UITableView, editActionsForRowAtIndexPath indexPath: NSIndexPath) -> [UITableViewRowAction]?  {
        let delete = UITableViewRowAction(style: UITableViewRowActionStyle.Default, title: "Delete" , handler: { (action:UITableViewRowAction, indexPath:NSIndexPath) -> Void in
            self.deleteItem(indexPath)
        })
        delete.backgroundColor = SwipeActionsColors.delete
        
        let duplicate = UITableViewRowAction(style: UITableViewRowActionStyle.Default, title: "Duplicate" , handler: { (action:UITableViewRowAction, indexPath:NSIndexPath) -> Void in
            //Do something
        })
        duplicate.backgroundColor = SwipeActionsColors.duplicate
        
        let edit = UITableViewRowAction(style: UITableViewRowActionStyle.Default, title: "Edit" , handler: { (action:UITableViewRowAction, indexPath:NSIndexPath) -> Void in
            self.editItem(indexPath)
        })
        edit.backgroundColor = SwipeActionsColors.edit
        
        return [delete, duplicate, edit]
    }
    
    /* --- DEBUG TOOLS --- */
    private func showDebugTools(){
        let btn1: UIButton = UIButton()
        btn1.frame = CGRectMake(0, 0, 280, 40)
        btn1.setTitle("Clean the database", forState: UIControlState.Normal)
        btn1.backgroundColor = UIColor.clearColor()
        btn1.setTitleColor(UIColor.orangeColor(), forState: UIControlState.Normal)
        btn1.addTarget(self, action: "cleanDatabase:", forControlEvents: UIControlEvents.TouchUpInside)
        
        let btnDeco: UIButton = UIButton()
        btnDeco.frame = CGRectMake(0, 50, 280, 40)
        btnDeco.setTitle("Run Sync Begin", forState: UIControlState.Normal)
        btnDeco.backgroundColor = UIColor.clearColor()
        btnDeco.setTitleColor(UIColor.orangeColor(), forState: UIControlState.Normal)
        btnDeco.addTarget(self, action: "runSyncBegin:", forControlEvents: UIControlEvents.TouchUpInside)
        
        let btnChange: UIButton = UIButton()
        btnChange.frame = CGRectMake(0, 100, 280, 40)
        btnChange.setTitle("Reset Settings", forState: UIControlState.Normal)
        btnChange.backgroundColor = UIColor.clearColor()
        btnChange.setTitleColor(UIColor.orangeColor(), forState: UIControlState.Normal)
        btnChange.addTarget(self, action: "resetSettings:", forControlEvents: UIControlEvents.TouchUpInside)
        
        let footerView: UIView = UIView(frame: CGRectMake(20, 0, 280, 150))
        footerView.addSubview(btn1)
        footerView.addSubview(btnDeco)
        footerView.addSubview(btnChange)
        tableView.tableFooterView = footerView
    }
    
    func cleanDatabase(sender: UIButton){
        let sb = SyncBegin()
        sb.deleteAll()
    }
    
    func runSyncBegin(sender: UIButton){
        baseActions!.runSyncBegin()
    }
    
    func resetSettings(sender: UIButton){
        Settings.resetSettings(true)
        viewDidLoad()
    }
    
    /* --- END DEBUG TOOLS --- */
}

