//
//  SettingsConstants.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 9/14/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import Foundation

// swift 1.2 does not support class variables -> need to use struct
struct SettingsConstants{
    static let USERNAME = "username_preference"
    static let PASSWORD = "password_preference"
    static let URL = "url_preference"
    static let SYNC_ENABLED = "sync_enabled_preference"
    static let SYNC_INTERVAL = "sync_interval_preference"
    static let NAME = "name_preference"
    static let PHONE = "phone_preference"
    static let EMAIL = "email_preference"
    static let DEBUG_MODE = "debug_mode_preference"
    
    static let LAST_LOGIN = "lastLogin"
    static let WELCOME_PAGE_DISMISSED = "welcomePageDismissed"
    static let DEVICE_ID = "deviceID"
}

public class Settings{

    class func getDefaults() -> NSUserDefaults{
        // hack: can not use `static let defaults: NSUserDefaults = ....`
        return NSUserDefaults.standardUserDefaults()
    }
    
    class func get(key: String)->AnyObject?{
        return getDefaults().valueForKey(key)
    }
    
    class func set(key: String, value: AnyObject){
        getDefaults().setValue(value, forKey: key)
    }
    
    class func resetSettings(overwriteExisting: Bool){
        /* set default settings values from config array */
        let settings = NSUserDefaults.standardUserDefaults()
        for (k, v) in config.defaultSettings {
            if (settings.objectForKey(k)==nil || overwriteExisting) {
                settings.setObject(v, forKey: k)
            }
        }
    }
    
    class var username: String? {
        get{
            return getDefaults().stringForKey(SettingsConstants.USERNAME)
        }
        set{
            getDefaults().setValue(newValue, forKey: SettingsConstants.USERNAME)
        }
    }
    
    class var password: String? {
        get{
            return getDefaults().stringForKey(SettingsConstants.PASSWORD)
        }
        set{
            getDefaults().setValue(newValue, forKey: SettingsConstants.PASSWORD)
        }
    }
    
    class var URL: String? {
        get{
            var result: String? = getDefaults().stringForKey(SettingsConstants.URL)
        
            // URL prefix must end by a slash
            if(result != nil && result! != "" && result!.substringFromIndex(result!.endIndex.predecessor()) != "/"){
                result!+="/"
            }
        
            return result
        }
        set{
            getDefaults().setValue(newValue, forKey: SettingsConstants.URL)
        }
    }
    
    class var syncEnabled: Bool {
        get{
        return getDefaults().boolForKey(SettingsConstants.SYNC_ENABLED)
        }
        set{
            getDefaults().setValue(newValue, forKey: SettingsConstants.SYNC_ENABLED)
        }
    }
    
    class var syncInterval: Float {
        get{
        return getDefaults().floatForKey(SettingsConstants.SYNC_INTERVAL)
        }
        set{
            getDefaults().setValue(newValue, forKey: SettingsConstants.SYNC_INTERVAL)
        }
    }
    
    class var name: String? {
        get{
        return getDefaults().stringForKey(SettingsConstants.NAME)
        }
        set{
            getDefaults().setValue(newValue, forKey: SettingsConstants.NAME)
        }
    }
    
    class var phone: String? {
        get{
        return getDefaults().stringForKey(SettingsConstants.PHONE)
        }
        set{
            getDefaults().setValue(newValue, forKey: SettingsConstants.PHONE)
        }
    }
    
    class var email: String? {
        get{
        return getDefaults().stringForKey(SettingsConstants.EMAIL)
        }
        set{
            getDefaults().setValue(newValue, forKey: SettingsConstants.EMAIL)
        }
    }
    
    class var debugMode: Bool {
        get{
        return getDefaults().boolForKey(SettingsConstants.DEBUG_MODE)
        }
        set{
            getDefaults().setValue(newValue, forKey: SettingsConstants.DEBUG_MODE)
        }
    }
    
    class var lastLogin: Bool {
        get{
        return getDefaults().boolForKey(SettingsConstants.LAST_LOGIN)
        }
        set{
            getDefaults().setValue(newValue, forKey: SettingsConstants.LAST_LOGIN)
        }
    }
    
    class var welcomePageDismissed: Bool {
        get{
        return getDefaults().boolForKey(SettingsConstants.WELCOME_PAGE_DISMISSED)
        }
        set{
            getDefaults().setValue(newValue, forKey: SettingsConstants.WELCOME_PAGE_DISMISSED)
        }
    }
    
    class var deviceID: Int? {
        get{
            if(getDefaults().objectForKey(SettingsConstants.DEVICE_ID) == nil){
                return nil
            }else{
                return getDefaults().integerForKey(SettingsConstants.DEVICE_ID)
            }
        }
        set{
            getDefaults().setValue(newValue, forKey: SettingsConstants.DEVICE_ID)
        }
    }
}

