//
//  ViewController.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 9/6/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import UIKit

protocol SyncBeginControllerDelegate{
    func syncBeginSuccessful()
    func badLogin(ctrl: SyncBeginController)
}

class SyncBeginController: UIViewController {
    
    var delegate:SyncBeginControllerDelegate? = nil
    
    /// If true, the received data are merged with the current database instead of replacing it. 
    var useMerge = false
    
    @IBOutlet weak var activityIndicator: UIActivityIndicatorView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // enlarge the preloader spinner
        activityIndicator.transform = CGAffineTransformMakeScale(1.5, 1.5)
    }
    
    override func viewWillAppear(animated: Bool) {
        runSyncBegin()
    }
    
    /**
     This is called from AppDelegate right before the app enters foreground. It is used for retrying sync begin agter bad login detected.
     */
    func applicationWillEnterForeground(){
        SyncQueue.getInstance().hideBadLoginDialog = false
        runSyncBegin()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    private func runSyncBegin(){
        NSLog("sync begin")
        
        let syncBegin = SyncBegin(useMerge: useMerge)
        
        syncBegin.successCallback = { () -> Void in
            NSLog("sync begin done")
            self.updateLastUsername()
            self.delegate?.syncBeginSuccessful()
            return Void()
        }
        
        syncBegin.badLoginCallback = { () -> Void in
            NSLog("bad login in sync begin")
            self.delegate?.badLogin(self)
            return Void()
        }
        
        syncBegin.run()
    }
    
    private func updateLastUsername(){
        let settings = NSUserDefaults.standardUserDefaults()
        let username = settings.stringForKey(SettingsConstants.USERNAME)
        settings.setValue(username, forKey: SettingsConstants.LAST_LOGIN)
    }
    
    @IBAction func cancelClick(sender: AnyObject) {
        let optionMenu = UIAlertController(title: l("Really stop downloading? Application data in your device may be damaged."), message: nil, preferredStyle: .ActionSheet)
        
        let action = UIAlertAction(title: "Stop", style: .Destructive, handler: {
            (alert: UIAlertAction!) -> Void in
            // @TODO stop downloading
            self.dismissViewControllerAnimated(true, completion: nil)
            
        })
        let cancelAction = UIAlertAction(title: "Cancel", style: .Cancel, handler: {
            (alert: UIAlertAction!) -> Void in
            
        })
        
        optionMenu.addAction(action)
        optionMenu.addAction(cancelAction)
        
        optionMenu.popoverPresentationController?.barButtonItem = sender as? UIBarButtonItem
        
        presentViewController(optionMenu, animated: true, completion: nil)
    }
}

