//
//  TempLayerAttributesManager.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 15/01/16.
//  Copyright © 2016 MENDELU. All rights reserved.
//

import Foundation

class TempLayerAttributesManager{
    
    var attributes: [TempLayerAttribute] = []
    var attributesToBeDeleted: [TempLayerAttribute] = []
    var storage: LayerAttributeStorage
    
    init(storage: LayerAttributeStorage){
        self.storage = storage
    }
    
    func load(layer: Layer){
        attributes = []
        let all = storage.getAllForLayer(layer).fetchedObjects as! [LayerAttribute]
        for attr in all{
            let tempAttr = TempLayerAttribute(title: attr.title, kind: attr.kind)
            tempAttr.dbObject = attr
            attributes.append(tempAttr)
        }
    }
    
    func create(new: TempLayerAttribute){
        attributes.append(new)
    }
    
    func delete(itemIndex: Int){
        let item = attributes[itemIndex]
        if(item.dbObject != nil){
            attributesToBeDeleted.append(item)
        }
        attributes.removeAtIndex(itemIndex)
    }
    
    func saveAttributesPernamentlyToLayer(layer: Layer){
        for item in attributes{
            if(item.dbObject == nil || item.dbObject?.managedObjectContext == nil){
                // create a new one in the central storage
                let attribute = storage.newEntity() as! LayerAttribute
                attribute.title = item.title
                attribute.kind = item.kind
                attribute.layer = layer
                storage.create(attribute)
            }else if(item.dbObject!.title != item.title || item.dbObject!.kind != item.kind){
                // update existing
                if(item.dbObject?.managedObjectContext != nil){
                    item.dbObject!.title = item.title
                    item.dbObject!.kind = item.kind
                    storage.update(item.dbObject!)
                }
            }
        }
        
        for item in attributesToBeDeleted{
            // delete existing
            storage.delete(item.dbObject!)
        }
    }
    
    func getAllAsString()->String{
        var str = ""
        for attr in attributes{
            str += attr.title + ", "
        }
        return str.substringToIndex(str.endIndex.advancedBy(-2))
    }
    
    func getByTitle(title: String)->TempLayerAttribute?{
        for attr in attributes{
            if(attr.title == title){
                return attr
            }
        }
        return nil
    }
}
