//
//  utils.swift
//  Gisella
//
//  Created by Ivo Pisarovic on 9/17/15.
//  Copyright (c) 2015 MENDELU. All rights reserved.
//

import Foundation
import SwiftyJSON
import CoreData

// MARK: - Strings

func l(txt: String)->String{
    return NSLocalizedString(txt, comment: "")
}

extension String{
    var trim: String {
        return self.stringByTrimmingCharactersInSet(NSCharacterSet.whitespaceCharacterSet())
    }
    
    public func indexOfCharacter(char: Character) -> Int? {
        if let idx = self.characters.indexOf(char) {
            return self.startIndex.distanceTo(idx)
        }
        return nil
    }
}

func makeJSONString(data: AnyObject?) -> String{
    if(data == nil){
        return ""
    }else{
        let inputJSON = JSON(data!)
        return makeJSONString(inputJSON)
    }
}

func makeJSONString(json: JSON?) -> String{
    if(json == nil){
        return ""
    }else{
        let result: String? = json!.rawString(NSUTF8StringEncoding, options: NSJSONWritingOptions.PrettyPrinted)
        return result!
    }
}

func stringToJSON(string: String)->JSON{
    if let dataFromString = string.dataUsingEncoding(NSUTF8StringEncoding, allowLossyConversion: false) {
        return JSON(data: dataFromString)
    }else{
        return nil
    }
}

func sanitizeURL(url: String)->String{
    let regex:NSRegularExpression = try! NSRegularExpression(pattern: "(\\/+)", options: NSRegularExpressionOptions.CaseInsensitive)
    let validURL = regex.stringByReplacingMatchesInString(url, options: [], range: NSMakeRange(0, (url.characters.count)), withTemplate: "/")
    return validURL
}

/** Converts input text to a string that can be safely used as a filename. */
func sanitizeFileName(text: String)->String{
    do{
        let regex = try NSRegularExpression(pattern: "[^a-zA-Z0-9_]", options: NSRegularExpressionOptions.CaseInsensitive)
        let filename = regex.stringByReplacingMatchesInString(text, options: NSMatchingOptions.ReportCompletion, range: NSMakeRange(0, text.characters.count), withTemplate: "_")
        return filename
    }catch{
        
    }
    return ""
}

// MARK: - Date

func dateToJSON(date: NSDate = NSDate())->String{
    let dateFormatter = NSDateFormatter()
    dateFormatter.dateFormat = "yyyy-MM-dd HH:mm:ss"
    return dateFormatter.stringFromDate(date)
}
func dateFromJSON(date: String)->NSDate{
    let dateFormatter = NSDateFormatter()
    dateFormatter.dateFormat = "yyyy-MM-dd HH:mm:ss"
    return dateFormatter.dateFromString(date)!
}

// MARK: - User Interface

func isIPad()->Bool{
    return UIDevice.currentDevice().userInterfaceIdiom == .Pad
}

// MARK: - Colors

func hexStringForColor(color: UIColor) -> String {
    let components = CGColorGetComponents(color.CGColor)
    let r: CGFloat = components[0]
    let g: CGFloat = components[1]
    let b: CGFloat = components[2]
    let hexString: String = "%02X%02X%02X"
    return hexString
}

// MARK: - Dialogs

func alert(msg: String, context: UIViewController? = nil){
    let alert = UIAlertController(title: "", message: msg, preferredStyle: UIAlertControllerStyle.Alert)
    alert.addAction(UIAlertAction(title: "OK", style: UIAlertActionStyle.Default, handler: { action in
    }))
    if(context != nil){
        context!.presentViewController(alert, animated: true, completion: nil)
    }else{
        showAlert(alert)
    }
}

func showAlert(alert: UIAlertController){
    if let ctrl = getTopViewController(){
        ctrl.presentViewController(alert, animated: true, completion: nil)
    }
}

func getTopViewController()->UIViewController?{
    if var topController = UIApplication.sharedApplication().keyWindow?.rootViewController {
        while let presentedViewController = topController.presentedViewController {
            topController = presentedViewController
        }
        return topController
    }
    return nil
}

func confirm(msg: String, context: UIViewController, sender: UIView, callback: ()->Void){
    let optionMenu = UIAlertController(title: msg, message: nil, preferredStyle: .ActionSheet)
    
    let action = UIAlertAction(title: "OK", style: .Destructive, handler: {
        (alert: UIAlertAction!) -> Void in
        callback()
    })
    let cancelAction = UIAlertAction(title: "Cancel", style: .Cancel, handler: {
        (alert: UIAlertAction!) -> Void in
        
    })
    
    optionMenu.addAction(action)
    optionMenu.addAction(cancelAction)

    optionMenu.popoverPresentationController?.sourceView = sender
    optionMenu.popoverPresentationController?.sourceRect = sender.bounds
    
    context.presentViewController(optionMenu, animated: true, completion: nil)
}

// MARK: - Core Data

func deleteAllEntities(entityName: String, moc: NSManagedObjectContext){
    // novejsi swift jiz ma na to metodu!!!
    let fetchRequest = NSFetchRequest()
    fetchRequest.entity = NSEntityDescription.entityForName(entityName, inManagedObjectContext: moc)
    fetchRequest.includesPropertyValues = false
    
    let error:NSError?
    do{
        let results = try moc.executeFetchRequest(fetchRequest) as? [NSManagedObject]
        for result in results! {
            moc.deleteObject(result)
        }
        
        do {
            try moc.save()
            // do something after save
            
        } catch let error1 as NSError {
            error = error1
            if let error = error {
                print(error.userInfo)
            }
        }
        
    } catch {
        print("error: \(error)")
    }
}

// MARK: - Log

func debug(msg: String){
    NSLog("DEBUG: %@", msg)
}

func error(msg: String){
    NSLog("ERROR: %@", msg)
}

func warning(msg: String){
    NSLog("WARNING: %@", msg)
}

// MARK: - Other

func abstract(){
    preconditionFailure("This method must be overridden!")
}

func getAppDelegate()->AppDelegate{
    return UIApplication.sharedApplication().delegate as! AppDelegate
}

extension UIImage {
    func imageWithColor(color1: UIColor, sizeX: CGFloat, sizeY: CGFloat) -> UIImage {
        UIGraphicsBeginImageContextWithOptions(self.size, false, self.scale)
        color1.setFill()
        
        let context = UIGraphicsGetCurrentContext()! as CGContextRef
        CGContextTranslateCTM(context, 0, self.size.height)
        CGContextScaleCTM(context, 1.0, -1.0);
        CGContextSetBlendMode(context, CGBlendMode.Normal)
        
        let rect = CGRectMake(0, 0, sizeX, sizeY/*self.size.width, self.size.height*/) as CGRect
        CGContextClipToMask(context, rect, self.CGImage)
        CGContextFillRect(context, rect)
        
        let newImage = UIGraphicsGetImageFromCurrentImageContext() as UIImage
        UIGraphicsEndImageContext()
        
        return newImage
    }
}
